Type: Package
Package: SingleCellMultiModal
Title: Integrating Multi-modal Single Cell Experiment datasets
Version: 1.25.3
Date: 2025-06-12
Authors@R: c(
    person("Marcel", "Ramos", , "marcel.ramos@sph.cuny.edu", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-3242-0582")),
    person("Ricard", "Argelaguet", , "ricard@ebi.ac.uk", role = "aut"),
    person("Al", "Abadi", role = "ctb"),
    person("Dario", "Righelli", , "dario.righelli@gmail.com", role = "aut"),
    person("Christophe", "Vanderaa", , "christophe.vanderaa@uclouvain.be", role = "ctb"),
    person("Kelly", "Eckenrode", , "kelly.eckenrode@sph.cuny.edu", role = "aut"),
    person("Ludwig", "Geistlinger", , "ludwig_geistlinger@hms.harvard.edu", role = "aut"),
    person("Levi", "Waldron", , "lwaldron.research@gmail.com", role = "aut"),
    person("NCI", role = "fnd",
           comment = c(GrantNo. = "U24CA289073"))
  )
Description: SingleCellMultiModal is an ExperimentHub package
    that serves multiple datasets obtained from GEO and other sources and
    represents them as MultiAssayExperiment objects. We provide several
    multi-modal datasets including scNMT, 10X Multiome, seqFISH, CITEseq,
    SCoPE2, and others. The scope of the package is is to provide data for
    benchmarking and analysis. To cite, use the 'citation' function and see 
    <https://doi.org/10.1371/journal.pcbi.1011324>.
License: Artistic-2.0
BugReports: https://github.com/waldronlab/SingleCellMultiModal/issues
Depends: R (>= 4.2.0), MultiAssayExperiment
Imports: AnnotationHub, BiocBaseUtils, BiocFileCache, ExperimentHub,
        graphics, HDF5Array, S4Vectors, SingleCellExperiment,
        SpatialExperiment, SummarizedExperiment, Matrix, methods, utils
Suggests: BiocStyle, ggplot2, knitr, RaggedExperiment, rmarkdown,
        scater, scran, UpSetR, uwot
VignetteBuilder: knitr
biocViews: ExperimentData, SingleCellData, ReproducibleResearch,
        ExperimentHub, GEO
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
Config/roxygen2/version: 8.0.0
git_url: https://git.bioconductor.org/packages/SingleCellMultiModal
git_branch: devel
git_last_commit: cdbd97d
git_last_commit_date: 2026-05-20
Repository: Bioconductor 3.24
Date/Publication: 2026-05-21
NeedsCompilation: no
Packaged: 2026-05-21 15:49:07 UTC; biocbuild
Author: Marcel Ramos [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3242-0582>),
  Ricard Argelaguet [aut],
  Al Abadi [ctb],
  Dario Righelli [aut],
  Christophe Vanderaa [ctb],
  Kelly Eckenrode [aut],
  Ludwig Geistlinger [aut],
  Levi Waldron [aut],
  NCI [fnd] (GrantNo.: U24CA289073)
Maintainer: Marcel Ramos <marcel.ramos@sph.cuny.edu>

# Generated by roxygen2: do not edit by hand

export(CITEseq)
export(GTseq)
export(SCoPE2)
export(SingleCellMultiModal)
export(addCTLabels)
export(getCellGroups)
export(ontomap)
export(removeCache)
export(scMultiome)
export(scNMT)
export(scmmCache)
export(seqFISH)
export(setCache)
import(MultiAssayExperiment)
importFrom(AnnotationHub,query)
importFrom(ExperimentHub,ExperimentHub)
importFrom(ExperimentHub,loadResources)
importFrom(Matrix,Matrix)
importFrom(MultiAssayExperiment,experiments)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(SingleCellExperiment,"altExp<-")
importFrom(SingleCellExperiment,"altExps<-")
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SingleCellExperiment,altExp)
importFrom(SingleCellExperiment,altExps)
importFrom(SingleCellExperiment,colData)
importFrom(SingleCellExperiment,counts)
importFrom(SpatialExperiment,SpatialExperiment)
importFrom(SummarizedExperiment,"assays<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(graphics,abline)
importFrom(graphics,smoothScatter)
importFrom(graphics,text)
importFrom(methods,is)
importFrom(utils,glob2rx)
importFrom(utils,read.csv)

---
output: github_document
knit: (function(inputFile, encoding) {
  rmarkdown::render(inputFile, encoding = encoding, output_dir = "../../") })
bibliography: ../REFERENCES.bib
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  cache = TRUE,
  out.width = "100%"
)
```

```{r,echo=FALSE,eval=FALSE}
## Generate the ./README.md (relative to base folder)
rmarkdown::render(input = "inst/scripts/README.Rmd", output_dir = ".")
```

# <a href='https://waldronlab.github.io/SingleCellMultiModal'><img src='https://raw.githubusercontent.com/Bioconductor/BiocStickers/devel/SingleCellMultiModal/SingleCellMultiModal.png' align="right" height="139" /></a>

```{r, child="../../vignettes/SingleCellMultiModal.Rmd"}

```

#' Obtain a map of cell types for each dataset
#'
#' The `ontomap` function provides a mapping of all the cell names across the
#' all the data sets or for a specified data set.
#'
#' @param dataset `character()` One of the existing functions within the
#'   package. If missing, a map of all cell types in each function will
#'   be provided.
#'
#' @details
#' Note that `CITEseq` does not have any cell annotations; therefore, no entries
#' are present in the `ontomap`.
#'
#' @return A `data.frame` of metadata with cell types and ontologies
#'
#' @examples
#'
#' ontomap(dataset = "scNMT")
#'
#' @export
ontomap <- function(
    dataset = c("scNMT", "scMultiome", "SCoPE2", "CITEseq", "seqFISH")
) {
    dataset <- match.arg(dataset, several.ok = TRUE)
    omap <- system.file(
        "extdata", "ontomap.tsv",
        package = "SingleCellMultiModal", mustWork = TRUE
    )
    map <- utils::read.delim(omap)
    dnames <- map[["function_name"]]
    map[dnames %in% dataset, ]
}

#' Single-cell Nucleosome, Methylation and Transcription sequencing
#'
#' @description scNMT assembles data on-the-fly from `ExperimentHub` to provide
#'   a
#'   [`MultiAssayExperiment`][MultiAssayExperiment::MultiAssayExperiment-class]
#'   container. The `DataType` argument provides access to the
#'   `mouse_gastrulation` dataset as obtained from Argelaguet et al. (2019; DOI:
#'   10.1038/s41586-019-1825-8). Pre-processing code can be seen at
#'   <https://github.com/rargelaguet/scnmt_gastrulation>. Protocol
#'   information for this dataset is available at Clark et al. (2018). See the
#'   vignette for the full citation.
#'
#' @details scNMT is a combination of RNA-seq (transcriptome) and an adaptation
#'   of Nucleosome Occupancy and Methylation sequencing (NOMe-seq, the
#'   methylome and chromatin accessibility) technologies. For more
#'   information, see Reik et al. (2018) DOI: 10.1038/s41467-018-03149-4
#'
#'  * mouse_gastrulation - this dataset provides cell quality control filters in
#'  the object `colData` starting from version 2.0.0. Additionally, cell types
#'  annotations are provided through the `lineage` `colData` column.
#'      * rna - RNA-seq
#'      * acc_\* - chromatin accessibility
#'      * met_\* - DNA methylation
#'          * cgi - CpG islands
#'          * CTCF - footprints of CTCF binding
#'          * DHS - DNase Hypersensitive Sites
#'          * genebody - gene bodies
#'          * p300 - p300 binding sites
#'          * promoter - gene promoters
#'
#'   Special thanks to Al J Abadi for preparing the published data in time
#'   for the 2020 BIRS Workshop, see the link here:
#'   <https://github.com/BIRSBiointegration/Hackathon/tree/master/scNMT-seq>
#'
#' @section versions:
#'   Version '1.0.0' of the scNMT mouse_gastrulation dataset includes all of
#'   the above mentioned assay technologies with filtering of cells based on
#'   quality control metrics. Version '2.0.0' contains all of the cells
#'   without the QC filter and does not contain CTCF binding footprints or
#'   p300 binding sites.
#'
#' @section metadata:
#'   The `MultiAssayExperiment` metadata includes the original function call
#'   that saves the function call and the data version requested.
#'
#' @param DataType `character(1)` Indicates study that produces this type of
#'   data (default: 'mouse_gastrulation')
#'
#' @param modes `character()` A wildcard / glob pattern of modes, such as
#'   `"acc*"`. A wildcard of `"*"` will return all modes including
#'   Chromatin Accessibilty ("acc"), Methylation ("met"), RNA-seq ("rna")
#'   which is the default.
#'
#' @param version `character(1)` Either version '1.0.0' or '2.0.0' depending on
#'   data version required (default '1.0.0'). See version section.
#'
#' @param dry.run `logical(1)` Whether to return the dataset names before actual
#'   download (default `TRUE`)
#'
#' @param verbose `logical(1)` Whether to show the dataset currently being
#'   (down)loaded (default `TRUE`)
#'
#' @param ... Additional arguments passed on to the
#'   \link[ExperimentHub]{ExperimentHub-class} constructor
#'
#' @seealso SingleCellMultiModal-package
#'
#' @return A single cell multi-modal
#'   [`MultiAssayExperiment`][MultiAssayExperiment::MultiAssayExperiment-class]
#'   or informative `data.frame` when `dry.run` is `TRUE`
#'
#' @source <http://ftp.ebi.ac.uk/pub/databases/scnmt_gastrulation/>
#'
#' @references
#'   Argelaguet et al. (2019)
#'
#' @examples
#'
#' scNMT(DataType = "mouse_gastrulation", modes = "*",
#'     version = "1.0.0", dry.run = TRUE)
#'
#' @export scNMT
scNMT <-
    function(
        DataType = "mouse_gastrulation", modes = "*", version = "1.0.0",
        dry.run = TRUE, verbose = TRUE, ...
    )
{
    stopifnot(.isSingleChar(version), .isSingleChar(DataType))
    meta <- list(call = match.call(), version = version)

    if (missing(version) || !version %in% c("1.0.0", "2.0.0"))
        stop("Enter version '1.0.0' or '2.0.0'; see '?scNMT' for details.")

    ess_list <- .getResourcesList(prefix = "scnmt_", datatype = DataType,
        modes = modes, version = version, dry.run = dry.run,
        verbose = verbose, ...)

    if (dry.run) { return(ess_list) }

    MultiAssayExperiment(
        experiments = ess_list[["experiments"]],
        colData = ess_list[["colData"]],
        sampleMap = ess_list[["sampleMap"]],
        metadata = meta
    )
}

#' Single-cell spatial + Gene Expression
#'
#' @description seqFISH function assembles data on-the-fly from `ExperimentHub`
#'   to provide a
#'   [`MultiAssayExperiment`][MultiAssayExperiment::MultiAssayExperiment-class]
#'   container. Actually the `DataType` argument provides access to the
#'   available datasets associated to the package.
#'
#' @details seq FISH data are a combination of single cell spatial coordinates
#'   and transcriptomics for a few hundreds of genes. seq-FISH data can be
#'   combined for example with scRNA-seq data to unveil multiple aspects of
#'   cellular behaviour based on their spatial organization and transcription.
#'
#' Available datasets are:
#' * mouse_visual_cortex: combination of seq-FISH data as obtained from Zhu
#' et al. (2018) and scRNA-seq data as obtained from Tasic et al. (2016),
#' Version 1.0.0 returns the full scRNA-seq data matrix, while version 2.0.0
#' returns the processed and subsetted scRNA-seq data matrix (produced for
#' the Mathematical Frameworks for Integrative Analysis of Emerging
#' Biological Data Types 2020 Workshop) The returned seqFISH data are always
#' the processed ones for the same workshop. Additionally, cell types
#' annotations are available in the `colData` through the `class` column in
#' the seqFISH `assay`.
#'     * scRNA_Counts - Tasic scRNA-seq gene count matrix
#'     * scRNA_Labels - Tasic scRNA-seq cell labels
#'     * seqFISH_Coordinates - Zhu seq-FISH spatial coordinates
#'     * seqFISH_Counts - Zhu seq-FISH gene counts matrix
#'     * seqFISH_Labels - Zhu seq-FISH cell labels
#'
#' @inheritParams scNMT
#'
#' @param DataType `character(1)` indicating the identifier of the dataset to
#'     retrieve.  (default "mouse_visual_cortex")
#'
#' @param modes `character()` The assay types or modes of data to obtain these
#'     include seq-FISH and scRNA-seq data by default.
#'
#' @return A
#'   [`MultiAssayExperiment`][MultiAssayExperiment::MultiAssayExperiment-class]
#'   of seq-FISH data
#'
#' @author Dario Righelli <dario.righelli <at> gmail.com>
#'
#' @importFrom SpatialExperiment SpatialExperiment
#' @importFrom SingleCellExperiment SingleCellExperiment
#' @importFrom S4Vectors DataFrame
#'
#' @examples
#'
#' seqFISH(DataType = "mouse_visual_cortex", modes = "*", version = "2.0.0",
#'     dry.run = TRUE)
#'
#' @export
seqFISH <-
    function(
        DataType="mouse_visual_cortex", modes="*", version,
        dry.run=TRUE, verbose=TRUE, ...
    )
{
    ess_list <- .getResourcesList(prefix = "seqfish_", datatype = DataType,
        modes = modes, version = version, dry.run = dry.run,
        verbose = verbose, ...)

    if (dry.run) { return(ess_list) }

    modes_list <- ess_list[["experiments"]]

    switch(DataType,
        "mouse_visual_cortex" = {
            mae <- .mouse_visual_cortex(modes_list=modes_list,
                                        version=version)
        },
        ## Add here other seqFISH datasets based on DataType identifier
        {
            stop("Unrecognized seqFISH dataset name")
        }
    )

    return(mae)
}

.mouse_visual_cortex <- function(modes_list, version)
{
    res <- paste0("scRNA",
        if (identical(version, "1.0.0")) "_Full" else "",
        "_", c("Counts", "Labels")
    )

    ## discrepancy between labels in counts and colData
    counts <- as.matrix(modes_list[[res[1]]])
    ## rowData is duplicate of rownames [removed]
    coldata <- modes_list[[res[2]]]
    vIDs <- intersect(rownames(coldata), colnames(counts))
    counts <- counts[, vIDs]
    coldata <- coldata[vIDs, ]

    sce <- SingleCellExperiment::SingleCellExperiment(
        colData=coldata,
        assays=S4Vectors::SimpleList(counts=counts)
    )

    se <- SpatialExperiment::SpatialExperiment(
        rowData=rownames(modes_list$seqFISH_Counts),
        colData=modes_list$seqFISH_Labels,
        assays=S4Vectors::SimpleList(
            counts=as.matrix(modes_list$seqFISH_Counts)),
        spatialData=DataFrame(modes_list$seqFISH_Coordinates),
        spatialCoordsNames=c("x", "y"))

    MultiAssayExperiment(
        experiments = list(seqFISH = se, scRNAseq = sce)
    )
}

.internalMap <- S4Vectors::DataFrame(
    FUN = c("scNMT", "scMultiome", "SCoPE2",
        "CITEseq", "CITEseq", "seqFISH", "GTseq"),
    DataType = c("mouse_gastrulation", "pbmc_10x",
        "macrophage_differentiation", "cord_blood",
        "peripheral_blood", "mouse_visual_cortex",
        "mouse_embryo_8_cell"
    )
)

.filterMap <- function(DataTypes, dry.run, verbose) {
    inDTypes <- match(DataTypes, .internalMap[["DataType"]])
    notfound <- is.na(inDTypes)
    if (any(notfound))
        stop("'", paste(DataTypes[notfound], collapse = ", "),
            "' is not available, ", "see ?SingleCellMultiModal")
    upmap <- .internalMap[inDTypes, , drop = FALSE]
    upmap[["dry.run"]] <- dry.run
    upmap[["verbose"]] <- verbose
    upmap
}

#' Combining Modalities into one MultiAssayExperiment
#'
#' Combine multiple single cell modalities into one using the input of the
#' individual functions.
#'
#' @inheritParams scNMT
#'
#' @param DataTypes `character()` A vector of data types as indicated in each
#'     individual function by the `DataType` parameter. These can be any of
#'     the following: "mouse_gastrulation", "pbmc_10x",
#'     "macrophage_differentiation", "cord_blood", "peripheral_blood",
#'     "mouse_visual_cortex", "mouse_embryo_8_cell"
#'
#' @param versions `character()` A vector of versions for each DataType. By
#'     default, version `1.0.0` is obtained for all data types.
#'
#' @param modes list() A list or CharacterList of modes for each data type
#'     where each element corresponds to one data type.
#'
#' @return A multi-modality `MultiAssayExperiment`
#'
#' @section metadata:
#'     The metadata in the `MultiAssayExperiment` contains the original
#'     function call used to generate the object (labeled as `call`),
#'     a `call_map` which provides traceability of technology functions to
#'     `DataType` prefixes, and lastly, R version information as `version`.
#'
#' @examples
#'
#' SingleCellMultiModal(c("mouse_gastrulation", "pbmc_10x"),
#'     modes = list(c("acc*", "met*"), "rna"),
#'     version = c("2.0.0", "1.0.0"), dry.run = TRUE, verbose = TRUE
#' )
#'
#' @export
SingleCellMultiModal <- function(
        DataTypes, modes = "*", versions = "1.0.0",
        dry.run = TRUE, verbose = TRUE, ...
    )
{
    stopifnot(is.character(DataTypes), is.character(versions))

    if (.isSingleChar(modes) && identical(modes, "*"))
        modes <- c(rep(modes, length(DataTypes)))
    if (.isSingleChar(versions) && identical(versions, "1.0.0"))
        versions <- c(rep(versions, length(DataTypes)))
    resmap <- .filterMap(DataTypes, dry.run, verbose)
    modes <- methods::as(modes, "CharacterList")
    resmap <- cbind(resmap, version = versions, modes = modes)
    meta <- list(call = match.call(), call_map = resmap, version = version)

    ess_lists <- apply(resmap, 1L,
        function(resrow) {
            if (verbose)
                message("Running ", resrow[[1]], "...")
            do.call(get(resrow[[1]]), resrow[-1])
        }
    )
    names(ess_lists) <- DataTypes

    if (dry.run) { return(ess_lists) }

    new_prefix <- paste0(resmap[["DataType"]], "_")
    ess_lists <- Map(function(x, y) {
        if (is(x, "MultiAssayExperiment"))
            names(x) <- paste0(y, names(x))
        x
    }, x = ess_lists, y = new_prefix)

    result <- Reduce(c, ess_lists)
    metadata(result) <- meta
    result
}

library(SingleCellExperiment)
library(DropletUtils)
cb <- CITEseq("cord_blood", dry.run=FALSE, DataClass="SingleCellExperiment")
adt <- SingleCellExperiment(assays=list(counts=assays(altExp(cb))[[1]]))
top.marker <- rownames(adt)[max.col(t(counts(adt)))]
total.count <- colSums(counts(adt))
boxplot(split(log10(total.count), top.marker), ylab="Log-total ADT count", las=2)
adt.counts <- counts(adt)
adt.detected <- colSums(adt.counts > 0)
hist(adt.detected, col='grey', main="", xlab="Number of detected ADTs")

qc.stats <- cleanTagCounts(adt)#, exclusive=c("CD3", "CD19"))
summary(qc.stats$high.ambient) # libraries removed with high ambient contamination

library(scater)
mito <- grep("mt-", tolower(rownames(adt)))
df <- perCellQCMetrics(adt, subsets=list(Mito=mito))
mito.discard <- isOutlier(df$subsets_Mito_percent, type="higher")
summary(mito.discard)

discard <- qc.stats$discard | mito.discard

colData(cb) <- cbind.DataFrame(colData(cb), adt.discard=qc.stats$discard, mito.discard=mito.discard, discard=discard)

scRNAseq_coldata <- as.data.frame(colData(cb))
dir.create("cord_blood/v1.0.0/", recursive=TRUE)
save(scRNAseq_coldata, file="cord_blood/v1.0.0/scRNAseq_coldata.rda")

## Alternatively it is possible to indicate two or more ADTs that should 
## be expressed alternatively in a cell.
## for CD3/CD4/CD8 i'm referring to https://tinyurl.com/ys9aawce
## otherwise the OSCA vignette 12.3.3 indicates to use CD3/CD19, but
## this article found that CD3and CD19 could be expressed in a novel cell type
## https://www.ncbi.nlm.nih.gov/pmc/articles/PMC8694500/
# qc.stats <- cleanTagCounts(adtsce1, exclusive=c("CD4", "CD8"))
# summary(qc.stats$discard) # libraries removed with high ambient contamination

library(SingleCellExperiment)
library(DropletUtils)
mae <- CITEseq("peripheral_blood", dry.run=FALSE)#, DataClass="SingleCellExperiment")
adt <- SingleCellExperiment(assays=list(counts=mae[["scADT"]]))
pb <-  SingleCellExperiment(assays=list(counts=mae[["scRNA"]]))
cn <- colnames(adt)
condition <- unlist(lapply(strsplit(colnames(adt), "_"), function(x) x[1]))
bc <- unlist(lapply(strsplit(colnames(adt), "_"), function(x) x[2]))
colData(adt) <- DataFrame("barcodes"=bc, "condition"=condition)
colnames(adt) <- cn

adt.rm <- adt[-c(3,52),]

adtcr <- adt.rm[, adt.rm$condition=="CTRL"]
adtcl <- adt.rm[, adt.rm$condition=="CTCL"]
top.markercr <- rownames(adtcr)[max.col(t(counts(adtcr)))]
top.markercl <- rownames(adtcl)[max.col(t(counts(adtcl)))]
total.countcr <- colSums(counts(adtcr))
total.countcl <- colSums(counts(adtcl))

boxplot(split(log10(total.countcr), top.markercr), ylab="Log-total ADT CTRL count", las=2) #CD5
boxplot(split(log10(total.countcl), top.markercl), ylab="Log-total ADT CTCL count", las=2) #CD279

adt.countscr <- counts(adtcr)
adt.detectedcr <- colSums(adt.countscr > 0)
hist(adt.detectedcr, col='grey', main="", xlab="Number of detected ADTs CTRL")

adt.countscl <- counts(adtcl)
adt.detectedcl <- colSums(adt.countscl > 0)
hist(adt.detectedcl, col='grey', main="", xlab="Number of detected ADTs CTCL")

qc.statscr <- cleanTagCounts(adtcr)#, exclusive=c("CD3", "CD19"))
summary(qc.statscr$high.ambient) # libraries removed with high ambient contamination

qc.statscl <- cleanTagCounts(adtcl)#, exclusive=c("CD3", "CD19"))
summary(qc.statscl$high.ambient) # libraries removed with high am

library(scater)
cn <- colnames(pb)
condition <- unlist(lapply(strsplit(colnames(pb), "_"), function(x) x[1]))
bc <- unlist(lapply(strsplit(colnames(pb), "_"), function(x) x[2]))
colData(pb) <- DataFrame("barcodes"=bc, "condition"=condition)
colnames(pb) <- cn
pbcr <- pb[,pb$condition=="CTRL"]
pbcl <- pb[,pb$condition=="CTCL"]
mito <- grep("mt-", tolower(rownames(pb)))
dfcr <- perCellQCMetrics(pbcr, subsets=list(Mito=mito))
dfcl <- perCellQCMetrics(pbcl, subsets=list(Mito=mito))
mito.discardcr <- isOutlier(dfcr$subsets_Mito_percent, type="higher")
names(mito.discardcr) <- rownames(dfcr)
summary(mito.discardcr)

mito.discardcl <- isOutlier(dfcl$subsets_Mito_percent, type="higher")
names(mito.discardcl) <- rownames(dfcl)
summary(mito.discardcl)

cd <- colData(mae)
cd
cd$adt.discard_CTRL <- FALSE
cd$adt.discard_CTRL[which(rownames(cd) %in% rownames(qc.statscr)[qc.statscr$discard])] <- TRUE
cd$adt.discard_CTCL <- FALSE
cd$adt.discard_CTCL[which(rownames(cd) %in% rownames(qc.statscl)[qc.statscl$discard])] <- TRUE

cd$mito.discard_CTRL <- FALSE
cd$mito.discard_CTRL[which(rownames(cd) %in% names(mito.discardcr)[mito.discardcr])] <- TRUE
cd$mito.discard_CTCL <- FALSE
cd$mito.discard_CTCL[which(rownames(cd) %in% names(mito.discardcl)[mito.discardcl])] <- TRUE
cd$discard_CTRL <- cd$adt.discard_CTRL | cd$mito.discard_CTRL
cd$discard_CTCL <- cd$adt.discard_CTCL | cd$mito.discard_CTCL
cd$discard <- cd$discard_CTRL | cd$discard_CTCL


scRNAseq_coldata <- as.data.frame(cd)
dir.create("peripheral_blood/v1.0.0/", recursive=TRUE)
save(scRNAseq_coldata, file="peripheral_blood/v1.0.0/scRNAseq_coldata.Rda")

.cord_blood <- function(ess_list) {
    experiments <- ess_list[["experiments"]]
    enames <- names(experiments)
    idx <- grep("Counts", enames)
    names(experiments) <- gsub("Counts|_Counts", "", enames)
    mae <- MultiAssayExperiment::MultiAssayExperiment(
        experiments = experiments[idx]
    )
    coldat <- sampleMap(mae)[, -c(1:2), drop = FALSE]
    rownames(coldat) <- coldat[, 1]
    colnames(coldat) <- "sampleID"
    cdataidx <- grep("coldata", enames)
    cd <- experiments[cdataidx][[1L]]
    ### check add clr counts
    if (!is.null(dim(cd))) {
        # colData(mae) <- S4Vectors::cbind.DataFrame(coldat, cd)
        colData(mae) <- DataFrame(cd)
    } else {
        colData(mae) <- coldat
    }
    return(mae)
}

.combMatrixForAssay <- function(
    explist,
    dimslist,
    assayId = c("scADT", "scHTO", "scRNA")
) {
    match.arg(assayId)
    assIdx <- grep(assayId, names(explist))
    switch(
        assayId,
        "scADT" = ,
        "scHTO" = {
            if (length(explist[assIdx]) == 2) {
                m1 <- Matrix::Matrix(
                    unlist(explist[assIdx]),
                    nrow = dimslist[assIdx][[1]][1],
                    ncol = (dimslist[assIdx][[1]][2] +
                        dimslist[assIdx][[2]][2]),
                    sparse = TRUE
                )
            } else {
                m1 <- Matrix::Matrix(explist[[assIdx]])
            }
        },
        "scRNA" = {
            if (length(explist[assIdx]) == 2) {
                ## we can have at last 2 matrices
                m1 <- cbind(explist[[assIdx[1]]], explist[[assIdx[2]]])
            } else {
                m1 <- explist[[assIdx]]
            }
        },
        {
            stop("Unrecognized assayId: ", assayId)
        }
    )
    if (length(explist[assIdx]) == 2) {
        colnames(m1) <- c(
            paste0(
                rep(
                    gsub("scADT|scHTO|scRNA", "", names(explist)[assIdx[1]]),
                    dimslist[assIdx][[1]][2]
                ),
                colnames(explist[[assIdx[1]]])
            ),
            paste0(
                rep(
                    gsub("scADT|scHTO|scRNA", "", names(explist)[assIdx[2]]),
                    dimslist[assIdx][[2]][2]
                ),
                colnames(explist[[assIdx[2]]])
            )
        )
        rownames(m1) <- rownames(explist[[assIdx[[1]]]])
    } else {
        colnames(m1) <- paste0(
            rep(
                gsub("scADT|scHTO|scRNA", "", names(explist)[assIdx[1]]),
                dimslist[assIdx][[1]][2]
            ),
            colnames(explist[[assIdx[1]]])
        )
        rownames(m1) <- rownames(explist[[assIdx[[1]]]])
    }
    return(m1)
}

.buildColData <- function(mat1, assayId) {
    cd <- DataFrame(
        colname = colnames(mat1),
        condition = gsub("_\\w+", "", colnames(mat1))
    )
    return(cd)
}

.buildMap <- function(mat1, assayId) {
    map <- DataFrame(
        assay = assayId,
        #primary=gsub("_\\w+", "", colnames(mat1)),
        primary = colnames(mat1),
        colname = colnames(mat1),
        condition = gsub("_\\w+", "", colnames(mat1))
    )
    return(map)
}

#' @importFrom Matrix Matrix
.peripheral_blood <- function(ess_list) {
    ll <- ess_list[["experiments"]]
    cdidx <- grep("coldata", names(ll))
    cd <- NULL
    if (length(cdidx) != 0) {
        cd <- ll[[cdidx]]
        ll <- ess_list[["experiments"]][-cdidx]
    }
    ll <- lapply(ll, function(x) {
        x <- x[order(rownames(x)), ]
    })

    dims <- lapply(ll, dim)
    # expslist <- vector("list", length(ll))
    # sampmap <- DataFrame()
    exps <- lapply(c("scADT", "scHTO", "scRNA"), function(assayn) {
        if (!isEmpty(grep(assayn, names(ll)))) {
            assmat <- .combMatrixForAssay(
                explist = ll,
                dimslist = dims,
                assayId = assayn
            )
            assmap <- .buildMap(assmat, assayId = assayn)
            return(list("EXP" = assmat, "SAMP" = assmap, "NAME" = assayn))
        }
    })
    names(exps) <- vapply(exps, `[[`, character(1L), "NAME")
    expslist <- lapply(exps, `[[`, "EXP")
    sampmap <- do.call("rbind", lapply(exps, `[[`, "SAMP"))
    if (is.null(cd)) {
        coldat <- .buildColData(ll)
        coldat <- sampmap[, -c(1:2)]
        colnames(coldat) <- c("sampleID", "condition")
        rownames(coldat) <- coldat[["sampleID"]]
        coldat <- unique(coldat)
    } else {
        coldat <- cd
    }
    mae <- MultiAssayExperiment::MultiAssayExperiment(
        experiments = expslist,
        sampleMap = sampmap,
        colData = coldat
    )
    if (!isEmpty(grep("TCR", names(ll)))) {
        metadata(mae) <- ll[grep("TCR", names(ll))]
    }
    return(mae)
}

#' CITEseq
#' @description function assembles data on-the-fly from `ExperimentHub` to
#'   provide a
#'   [`MultiAssayExperiment`][MultiAssayExperiment::MultiAssayExperiment-class]
#'   container. Actually the `dataType` argument provides access to the
#'   available datasets associated to the package.
#' @author Dario Righelli
#' @details CITEseq data are a combination of single cell transcriptomics and
#'   about a hundread of cell surface proteins.
#'   Available datasets are:
#'   * cord_blood: a dataset of single cells of cord blood as
#'   provided in Stoeckius et al. (2017).
#'      * scRNA_Counts - Stoeckius scRNA-seq gene count matrix
#'      * scADT - Stoeckius antibody-derived tags (ADT) data
#'   * peripheral_blood: a dataset of single cells of peripheral
#'   blood as provided in Mimitou et al. (2019). We provide two different
#'   conditions controls (CTRL) and Cutaneous T-cell Limphoma (CTCL). Just build
#'   appropriate `modes` regex for subselecting the dataset modes.
#'      * scRNA - Mimitou scRNA-seq gene count matrix
#'      * scADT - Mimitou antibody-derived tags (ADT) data
#'      * scHTO - Mimitou Hashtag Oligo (HTO) data
#'      * TCRab - Mimitou T-cell Receptors (TCR) alpha and beta
#'      available through the object metadata.
#'      * TCRgd - Mimitou T-cell Receptors (TCR) gamma and delta
#'      available through the object metadata.
#'
#' @param DataType `character(1)` indicating the identifier of the dataset to
#'     retrieve.  (default "cord_blood")
#'
#' @param modes `character()` The assay types or modes of data to obtain these
#'     include scADT and scRNA-seq data by default.
#'
#' @param version `character(1)` Either version '1.0.0' depending on
#'     data version required.
#' @param dry.run `logical(1)` Whether to return the dataset names before actual
#'     download (default `TRUE`)
#' @param filtered `logical(1)` indicating if the returned dataset needs to
#'     have filtered cells.
#'     See Details for additional information about the filtering process.
#'
#' @param verbose `logical(1)` Whether to show the dataset currently being
#'     (down)loaded (default `TRUE`)
#'
#' @param ... Additional arguments passed on to the
#'     \link[ExperimentHub]{ExperimentHub-class} constructor
#'
#' @param DataClass either MultiAssayExperiment or SingleCellExperiment
#' data classes can be returned (default MultiAssayExperiment)
#'
#' @details
#' If `filtered` parameter is `FALSE` (default), the `colData` of the returned
#' object contains multiple columns of `logicals` indicating the cells to be
#' discarded.
#' In case `filtered` is `TRUE`, the `discard` column is used to filer the
#' cells.
#' Column `adt.discard` indicates the cells to be discarded computed on the ADT
#' assay.
#' Column `mito.discard` indicates the cells to be discarded computed on the
#' RNA assay and mitocondrial genes.
#' Column `discard` combines the previous columns with an `OR` operator.
#' Note that for the `peripheral_blood` dataset these three columns are
#' computed and returned separately for the `CTCL` and `CTRL` conditions.
#' In this case the additional `discard` column combines the `discard.CTCL` and
#' `discard.CTRL` columns with an `OR` operator.
#' Cell filtering has been computed for `cord_blood` and `peripheral_blood`
#' datasets following section 12.3 of the Advanced Single-Cell Analysis with
#' Bioconductor book.
#' Executed code can be retrieved in the CITEseq_filtering.R script of this
#' package.
#'
#' @return A single cell multi-modal
#'   [`MultiAssayExperiment`][MultiAssayExperiment::MultiAssayExperiment-class]
#'   or informative `data.frame` when `dry.run` is `TRUE`. When `DataClass` is
#'   `SingleCellExperiment` an object of this class is returned with an RNA
#'   assay as main experiment and other assay(s) as `AltExp(s)`.
#' @references Stoeckius et al. (2017), Mimitou et al. (2019)
#' @export
#'
#' @examples
#'
#' mae <- CITEseq(DataType="cord_blood", dry.run=FALSE)
#' experiments(mae)
CITEseq <- function(
    DataType = c("cord_blood", "peripheral_blood"),
    modes = "*",
    version = "1.0.0",
    dry.run = TRUE,
    filtered = FALSE,
    verbose = TRUE,
    DataClass = c("MultiAssayExperiment", "SingleCellExperiment"),
    ...
) {
    dataType <- match.arg(DataType)
    message("Dataset: ", dataType)
    dataClass <- match.arg(DataClass)
    ess_list <- .getResourcesList(
        prefix = "citeseq_",
        datatype = dataType,
        modes = modes,
        version = version,
        dry.run = dry.run,
        verbose = verbose,
        ...
    )
    if (!dry.run) {
        mae <- switch(
            dataType,
            "cord_blood" = {
                .cord_blood(ess_list = ess_list)
            },
            "peripheral_blood" = {
                .peripheral_blood(ess_list = ess_list)
            },
            ## Add here other CITE-seq datasets based on DataType identifier
            {
                stop("Unrecognized CITE-seq dataset name: ", DataType)
            }
        )
        if (filtered) {
            sampleMap(mae) <- sampleMap(mae)[!colData(mae)$discard, ]
        }
        if (dataClass == "SingleCellExperiment") return(.CITEseqMaeToSce(mae))
        return(mae)
    } else {
        return(ess_list)
    }
}


#' CITEseqMaeToSce
#' @description converts a `MultiAssayExperiment` object with CITEseq data into
#' a `SingleCellExperiment` object to be used with already known methods and
#' packages in literature.
#'
#' Note that for creating a `SingleCellExperiment` object the following function
#' subsets all the assays present in the `MultiAssayExperiment` with only the
#' common cells across all the modalities.
#' This could result in a not complete object.
#'
#'
#' @param mae a MultiAssayExperiment object with scRNA and/or scADT and/or
#' scHTO named experiments.
#'
#' @return a SingleCellExperiment object as widely with scRNA data as counts
#' and scADT, scHTO data as altExps.
#' If only one modality is present, it has returned as main assay of the SCE.
#'
#' @importFrom MultiAssayExperiment experiments
#' @importFrom SummarizedExperiment SummarizedExperiment assays<-
#' @importFrom SingleCellExperiment SingleCellExperiment altExp altExp<- altExps
#' @importFrom SingleCellExperiment altExps<- colData counts
#' @importFrom methods is
#' @importFrom S4Vectors SimpleList
#' @keywords internal
.CITEseqMaeToSce <- function(mae) {
    stopifnot(c(is(mae, "MultiAssayExperiment"), !(length(mae) == 0)))

    cs <- colnames(mae[[1]])
    for (i in seq_along(mae)[-1]) {
        cs <- intersect(cs, colnames(mae[[i]]))
    }

    scelist <- lapply(seq_along(mae), function(i) {
        sce <- SingleCellExperiment(list(counts = mae[[i]]))
        sce <- sce[, (colnames(sce) %in% cs)]
        cd <- colData(mae)[(rownames(colData(mae)) %in% colnames(sce)), ]
        colData(sce) <- cd
        return(sce)
    })
    names(scelist) <- names(mae)

    idx <- grep("scRNA", names(scelist))
    if (length(idx) != 0) {
        altExps(scelist[[idx]]) <- scelist[-idx]
        sce <- scelist[[idx]]
    } else {
        stop("Couldn't find RNA assay in MultiAssayExperiment")
    }
    idx <- grep("scADT_clr", names(altExps(sce)))
    if (length(idx) != 0) {
        clr <- counts(altExps(sce)[[idx]])
        altExps(sce)[idx] <- NULL
        assays(altExp(sce)) <- SimpleList(
            counts = counts(altExp(sce)),
            clr = clr
        )
    }

    if (!isEmpty(metadata(mae))) {
        metadata(sce) <- metadata(mae)
    }

    return(sce)
}

.getCache <- function() {
    cache <- getOption("scmmCache", setCache(verbose = FALSE))
    BiocFileCache::BiocFileCache(cache)
}

#' @name scmmCache
#'
#' @title Manage cache / download directories for study data
#'
#' @description Managing data downloads is important to save disk space and
#' re-downloading data files. This can be done effortlessly via the integrated
#' `BiocFileCache` system.
#'
#' @section scmmCache:
#' Get the directory location of the cache. It will prompt the user to create
#' a cache if not already created. A specific directory can be used via
#' `setCache`.
#'
#' @section setCache:
#' Specify the directory location of the data cache. By default, it will
#' go into the user's home and package name directory as given by
#' [R_user_dir][tools::R_user_dir] (default: varies by system e.g., for Linux:
#' '$HOME/.cache/R/SingleCellMultiModal').
#'
#' @section removeCache:
#' Some files may become corrupt when downloading, this function allows
#' the user to delete the tarball associated with a study number in the
#' cache.
#'
#' @param directory `character(1)` The file location where the cache is located.
#' Once set, future downloads will go to this folder. See `setCache` section
#' for details.
#'
#' @param verbose Whether to print descriptive messages
#'
#' @param ask `logical(1)` (default TRUE when `interactive()`) Confirm the file
#' location of the cache directory
#'
#' @param accession `character(1)` A single string indicating the accession number
#' of the study
#'
#' @param ... For `scmmCache`, arguments passed to `setCache`
#'
#' @examples
#' getOption("scmmCache")
#' scmmCache()
#'
#' @return The directory / option of the cache location
#'
#' @export
scmmCache <- function(...) {
    getOption("scmmCache", setCache(..., verbose = FALSE))
}

#' @rdname scmmCache
#' @export
setCache <-
    function(directory = tools::R_user_dir("SingleCellMultiModal", "cache"),
        verbose = TRUE,
        ask = interactive())
{
    stopifnot(
        is.character(directory), length(directory) == 1L, !is.na(directory)
    )

    if (!dir.exists(directory)) {
        if (ask) {
            qtxt <- sprintf(
                "Create cBioPortalData cache at \n    %s? [y/n]: ",
                directory
            )
            answer <- .getAnswer(qtxt, allowed = c("y", "Y", "n", "N"))
            if ("n" == answer)
                stop("'cbioCache' directory not created. Use 'setCache'")
        }
        dir.create(directory, recursive = TRUE, showWarnings = FALSE)
    }
    options("cbioCache" = directory)

    if (verbose)
        message("cBioPortalData cache directory set to:\n    ",
                directory)
    invisible(directory)
}

#' @rdname scmmCache
#' @export
removeCache <- function(accession) {
    bfc <- .getCache()
    rid <- BiocFileCache::bfcquery(bfc, accession, "rname", exact = TRUE)$rid
    if (length(rid)) {
        BiocFileCache::bfcremove(bfc, rid)
        message("Cache record: ", accession, ".tar.gz removed")
    } else
        message("No record found: ", accession, ".tar.gz")
}

.getAnswer <- function(msg, allowed)
{
    if (interactive()) {
        repeat {
            cat(msg)
            answer <- readLines(n = 1)
            if (answer %in% allowed)
                break
        }
        tolower(answer)
    } else {
        "n"
    }
}

.isSingleChar <- function(x) {
    length(x) == 1L && is.character(x) && !is.na(x)
}

.removeExt <- function(fnames) {
    gsub("\\..*$", "", basename(fnames))
}

.modesAvailable <- function(listfiles, prefix) {
    slots <- c("metadata", "colData", "sampleMap")
    modes <- gsub(prefix, "", listfiles, fixed = TRUE)
    modes <- gsub("_assays|_se|_tenx", "", modes)
    modes <- .removeExt(modes)
    unique(sort(modes[!modes %in% slots]))
}

.searchFromInputs <- function(glob, searchFields) {
    regGlob <- glob2rx(unique(glob))
    res <- unlist(lapply(regGlob, function(x) {
        grep(x, searchFields, ignore.case = TRUE, value = TRUE)
        }))
    if (!length(res))
        stop("No matches found, modify search criteria")
    res
}

.conditionToIndex <- function(startVec, testVec, FUN) {
    logmat <- vapply(startVec, FUN, logical(length(testVec)))
    apply(logmat, 1L, any)
}

.queryResources <- function(ExperimentHub, resTable, verbose) {
    fileNames <- stats::setNames(resTable[["RDataPath"]], resTable[["Title"]])
    lapply(fileNames, function(res) {
        if (verbose)
            message("Working on: ", gsub("\\.rda", "", basename(res)))
        # only take the last one for multiple matches
        utils::tail(query(ExperimentHub, res), 1)
    })
}

.getResources <- function(ExperimentHub, resTable, prefix, verbose) {
    infos <- .queryResources(ExperimentHub, resTable, verbose)
    rpath <- vapply(infos, function(x) `$`(x, "rdatapath"), character(1L))

    h5resources <- grepl("\\.[Hh]5$", rpath)
    mtxresources <- grepl("\\.[Mm][Tt][Xx]\\.[Gg][Zz]$", rpath)
    shells <- grepl("se\\.[Rr][Dd][Ss]$", rpath)
    otherres <- !((h5resources | mtxresources) | shells)

    if (any(h5resources))
        matress <- .loadHDF5(ExperimentHub, rpath, verbose)
    else if (any(mtxresources))
        matress <- .loadMTX(ExperimentHub, rpath, verbose)
    else
        matress <- list()

    if (any(otherres)) {
        rest <- lapply(infos[otherres], `[[`, 1L)
        c(rest, matress)
    } else {
        matress
    }
}

.getResourceInfo <- function(ExperimentHub, resTable, prefix, verbose) {
    infos <- .queryResources(ExperimentHub, resTable, verbose)
    resID <- vapply(infos, names, character(1L))
    restab <- AnnotationHub::getInfoOnIds(ExperimentHub, resID)
    restab <-
        restab[, !names(restab) %in% c("fetch_id", "status", "biocversion")]
    sizes <- as.numeric(restab[["file_size"]])
    class(sizes) <- "object_size"
    titleidx <- which(names(restab) == "title")
    restab <- as.data.frame(append(
        restab,
        list(mode = gsub(prefix, "", restab[["title"]]),
            file_size = format(sizes, units = "Mb")),
        titleidx
    ))
    restab[, -c(length(restab), titleidx)]
}

.test_eh <- function(...) {
    tryCatch({
        ExperimentHub(...)
    }, error = function(e) {
        emsg <- conditionMessage(e)
        if (grepl("Timeout", emsg))
            warning("[experimenthub.bioconductor.org] timeout, localHub=TRUE",
                call.=FALSE)
        ExperimentHub(..., localHub = TRUE)
    })
}

.isSingleCharNA <- function(x) {
    is.character(x) && length(x) == 1L && !is.na(x)
}

.getResourcesList <-
    function(prefix, datatype, modes, version, format, dry.run, verbose, ...)
{
    modes_file <- system.file("extdata", "metadata.csv",
        package = "SingleCellMultiModal", mustWork = TRUE)

    DataType <- tolower(datatype)
    stopifnot(
        .isSingleCharNA(DataType), .isSingleCharNA(version)
    )

    modes_metadat <- read.csv(modes_file, stringsAsFactors = FALSE)
    if (missing(format))
        notfmt <- "FakeFormatNoMatch"
    else
        notfmt <- switch(format, HDF5 = "MTX", MTX = "HDF5", format)
    filt <- modes_metadat[["DataType"]] == DataType &
        modes_metadat[["SourceVersion"]] == version &
        modes_metadat[["SourceType"]] != notfmt

    modes_metadat <- modes_metadat[filt, , drop = FALSE]
    eh_assays <- modes_metadat[["ResourceName"]]
    modesAvail <- .modesAvailable(eh_assays, prefix)
    resultModes <- .searchFromInputs(modes, modesAvail)
    fileIdx <- .conditionToIndex(
        resultModes, eh_assays, function(x) grepl(x, eh_assays)
    )
    fileMatches <-
        modes_metadat[fileIdx, c("Title", "DispatchClass", "SourceVersion")]
    eh <- .test_eh(...)

    if (dry.run) {
        return(.getResourceInfo(
            eh, modes_metadat[fileIdx, c("Title", "RDataPath")], prefix, FALSE
        ))
    }
    modes_list <- .getResources(
        eh, modes_metadat[fileIdx, c("Title", "RDataPath")], prefix, verbose
    )
    names(modes_list) <- gsub(prefix, "", names(modes_list))

    eh_experiments <- ExperimentList(modes_list)[resultModes]

    ess_names <- c("colData", "metadata", "sampleMap")

    ess_idx <- .conditionToIndex(ess_names, eh_assays,
        function(x) grepl(x, eh_assays))

    ess_list <- .getResources(eh,
        modes_metadat[ess_idx, c("Title", "RDataPath")], prefix, verbose)
    names(ess_list) <- gsub(prefix, "", names(ess_list))

    c(list(experiments = eh_experiments), ess_list)
}

.mergeLowColData <- function(x) {
    newcoldata <- Reduce(
        function(x, y) {
            S4Vectors::merge(x, y, by = "row.names", all = TRUE)
        },
        lapply(x, colData)
    )
    if (length(x) > 1L) {
        rownames(newcoldata) <- newcoldata[["Row.names"]]
        newcoldata <- newcoldata[, -which(colnames(newcoldata) == "Row.names")]
    }
    newcoldata
}

##
## PLACEHOLDER for make-data/
##
## see the respective R script for each technology
##
## for example, make-data/scNMT.R for an example on how
## we took a MultiAssayExperiment object and created
## Rda files for upload to ExperimentHub
##
## see make-data/scMultiome.R for converting a MultiAssayExperiment
## into sparceMatrix representations using HDF5 and MTX formats
##
## see make-data/make-macrophage.R for taking raw data and
## creating SingleCellExperiment data products
##

setwd("~/gh/SingleCellMultiModal")

.getSourceType <- function(filepaths) {
    lfiles <- strsplit(basename(filepaths), "\\.")
    exts <- vapply(lfiles,
        function(x) { paste(x[-1], collapse = ".") }, character(1L))
    uexts <- toupper(exts)
    uexts <- gsub("[Hh]5", "HDF5", uexts)
    uexts <- gsub("[Mm][Tt][Xx]\\.[Gg][Zz]", "MTX", uexts)
    vTypes <- AnnotationHubData::getValidSourceTypes()
    uTypes <- toupper(vTypes)
    allvalid <- all(uexts %in% uTypes)
    if (!allvalid)
        stop("Source types not supported: ", paste0(exts[!allvalid],
            collapse = ", "), "\n See 'AnnotationHubData::getValidSources()'",
            call. = FALSE)
    res <- vTypes[match(uexts, uTypes)]
    ## hot fix before AnnotationHubData 1.21.2
    gsub("MTX", "mtx.gz", res, fixed = TRUE)
}

doc_helper <-
    function(
        DataProvider, TaxonomyId, Species, SourceUrl, SourceType, DataType, ...
    )
{
    args <- list(...)
    saf <- args[["stringsAsFactors"]]
    saf <- if(!is.null(saf)) saf else FALSE

    input_vals <- list(
        DataProvider = DataProvider, TaxonomyId = TaxonomyId,
        Species = Species, SourceUrl = SourceUrl,
        SourceType = SourceType, DataType = DataType
    )
    clens <- lengths(input_vals)
    zlen <- !clens
    if (any(zlen))
        stop(
            "Provide values for: ",
            paste(names(input_vals)[zlen], collapse = ", ")
        )

    nonstd <- !clens %in% c(max(clens), 1L)
    if (any(nonstd))
        stop("Lengths of inputs must either be 1 or the max length")

    input_vals[clens == 1L] <- lapply(input_vals[clens == 1L],
        function(x) {
            rep(x, max(clens))
        })

    as.data.frame(input_vals, stringsAsFactors = saf)
}

.stdLength <- function(metalist, replength) {
    lapply(metalist, function(field) {
        if (length(field) == 1L)
            rep(field, replength)
        else
            field
    })
}

.loadRDS <- function(filepath) {
    readRDS(filepath)
}

.loadRDA <- function(filepath) {
    basefile <- gsub("\\.[Rr][Dd][Aa]", "", basename(filepath))
    OBJENV <- new.env(parent = emptyenv())
    load(filepath, envir = OBJENV)
    OBJENV[[basefile]]
}

.loadH5 <- function(filepath) {
    if (grepl("tenx", filepath))
        HDF5Array::TENxMatrix(filepath, "pbmc")
    else
        HDF5Array::HDF5Array(filepath, "assay001")
}

.loadMTX.GZ <- function(filepath) {
    .read_mtx(filepath)
}

.loadDataList <- function(filepaths) {
    recipelist <- list(
        "\\.[Rr][Dd][Aa]" = .loadRDA,
        "\\.[Rr][Dd][Ss]" = .loadRDS,
        "\\.[Hh]5" = .loadH5,
        "\\.[Mm][Tt][Xx]\\.[Gg][Zz]" = .loadMTX.GZ
    )
    hitMatrix <- vapply(names(recipelist),
        function(pat) grepl(pat, filepaths),
        logical(length(filepaths))
    )
    allrecipes <- recipelist[apply(hitMatrix, 1L, which)]
    Map(function(x, y) { x(y) }, x = allrecipes, y = filepaths)
}

any.na <- function(x) {
    any(is.na(x))
}

.get_Description <- function(data_name, DataType) {
    paste(data_name, "data specific to the", toupper(DataType), "project")
}

.getRDataClass <- function(dataList) {
    vapply(dataList, function(dataName) {
            if (is.matrix(dataName))
                "matrix"
            else
                class(dataName)
    }, character(1L))
}


.file_pattern_map <- data.frame(
    ext_pattern = paste0(
        c("[Rr][Dd][Aa]", "[Rr][Dd][Ss]", "[Hh]5", "[Mm][Tt][Xx]\\.[Gg][Zz]"),
        "$"
    ),
    ## currently MTX DispatchClass recipe unavailable
    Dispatch = c("Rda", "Rds", "H5File", "FilePath"),
    stringsAsFactors = FALSE
)

.getDispatchClass <- function(resource_files, ext_map = .file_pattern_map) {
    hitMatrix <- vapply(ext_map[["ext_pattern"]],
        function(pat) grepl(pat, resource_files),
            logical(length(resource_files)))
    ext_map[["Dispatch"]][apply(hitMatrix, 1L, which)]
}

## alist() with formals()<-
## fancyFUN <- function() {}
## formals(fancyFUN) <- alist()

MetaHubCreate <-
    function(base_dir, data_dirs, ext_pattern, doc_file, version, pkg_name)
{
    locations <- file.path(base_dir, data_dirs, paste0("v", version))
    stopifnot(
        dir.exists(base_dir), all(dir.exists(locations)),
        is.character(ext_pattern), !is.na(ext_pattern),
        identical(length(ext_pattern), 1L),
        file.exists(doc_file), is.character(doc_file), !is.na(doc_file),
        identical(length(doc_file), 1L), is.character(version)
    )
    fpathlist <- lapply(locations, function(locs) {
        list.files(
            locs, pattern = ext_pattern, full.names = TRUE, recursive = TRUE
        )
    })
    docFrame <- read.csv(doc_file, header = TRUE)
    docList <- split(docFrame,
        list(docFrame[["DataType"]], docFrame[["SourceVersion"]]))
    versions <- version
    DataTypes <- data_dirs
    replengths <- lengths(fpathlist)
    namelist <- lapply(fpathlist, basename)

    metaList <- Map(
        function(DataType, doc_file, resnames, filepaths, replength, version) {
            message("Working on: ", basename(DataType), " v", version)
            hubmeta <- R6::R6Class("EHubMeta",
                public = list(
                    Title = NA_character_,
                    Description = NA_character_,
                    BiocVersion = as.character(BiocManager::version()),
                    Genome = NA_character_,
                    SourceType = NA_character_,
                    SourceUrl = character(1L),
                    SourceVersion = version,
                    Species = character(1L),
                    TaxonomyId = character(1L),
                    Coordinate_1_based = NA,
                    DataProvider = character(1L),
                    Maintainer = NA_character_,
                    RDataClass = NA_character_,
                    DispatchClass = .getDispatchClass(resnames),
                    Location_Prefix = NA_character_,
                    RDataPath = NA_character_,
                    ResourceName = resnames,
                    DataType = DataType,

                    initialize = function(doc_file)
                    {
                        lapply(names(doc_file), function(i) {
                            assign(i, doc_file[[i]], self)
                        })
                        if (is.na(self$Title))
                            self$Title <- gsub(ext_pattern, "",
                                basename(filepaths))
                        if (is.na(self$Description))
                            self$Description <- paste(self$Title,
                                "data specific to the", toupper(self$DataType),
                                "project")
                        if (any.na(self$SourceType))
                            self$SourceType <- .getSourceType(filepaths)
                        if (any.na(self$SourceVersion))
                            self$SourceVersion <- "1.0.0"
                        if (any.na(self$Maintainer))
                            self$Maintainer <- utils::maintainer(pkg_name)
                        if (any.na(self$RDataClass)) {
                            dataList <- .loadDataList(filepaths)
                            self$RDataClass <- .getRDataClass(dataList)
                        }
                        if (is.na(self$Location_Prefix))
                            self$Location_Prefix <- NULL
                        if (is.na(self$RDataPath))
                            self$RDataPath <- file.path(pkg_name,
                                self$DataType, paste0("v", version),
                                self$ResourceName)
                    },
                    generate = function() {
                        lnames <- !names(self) %in%
                            c(".__enclos_env__", "clone", "generate",
                                "initialize")
                        initList <- mget(names(self)[lnames], envir = self)
                        initList <- Filter(function(x) !is.null(x), initList)
                        flist <- .stdLength(initList, replength)
                        do.call(data.frame, c(flist, stringsAsFactors = FALSE))
                    }
                ),
                lock_objects = FALSE
            )
            nhub <- hubmeta$new(doc_file)
            nhub$generate()
    }, DataType = DataTypes, doc_file = docList, resnames = namelist,
    filepaths = fpathlist, replength = replengths, version = versions
    )

    do.call(
        function(...) {
            rbind.data.frame(..., make.row.names = FALSE,
                stringsAsFactors = FALSE)
        },
    metaList)
}

#' Generate the metadata.csv file from a documentation file
#'
#' This function takes a specific folder structure and generates the
#' metadata.csv file for adding to ExperimentHub.
#'
#' @param directory The base folder for _all_ datasets
#'
#' @param dataDirs `character()` A vector of folder names contained in directory
#'     that corresponds to each project. For multiple versions, repeat the
#'     name of the folder.
#'
#' @param version `character()` A vector of subfolder versions that is parallel
#'     to `dataDirs` argument, typically `v1.0.0`.
#'
#' @param ext_pattern `character(1)` A string that matches files within the
#'     above folders to find the data.
#'
#' @param doc_file `character(1)` A path to the documentation `data.frame` that
#'     tells the function how to fill in the standard columns for data
#'     annotation, for example `DataProvider`, `TaxonomyId`, etc.
#'
#' @param pkg_name `character(1)` The name of the current package
#'
#' @param dry.run `logical(1)` Whether to (over)write the `metadata.csv` file or
#'     return as output.
#'
#' @param append `logical(1)` Whether to append to the current `metadata.csv`
#'     file
#'
#' @return Saves a file under `/inst/extdata/metadata.csv`
#'
#' @examples
#'
#' make_metadata(
#'     directory = "~/data/scmm",
#'     dataDirs = "mouse_gastrulation",
#'     version = c("1.0.0", "2.0.0"),
#'     doc_file = "inst/extdata/docuData/singlecellmultimodalv2.csv",
#'     dry.run = FALSE
#' )
#'
#' make_metadata(
#'     directory = "~/data/scmm",
#'     dataDirs = c(rep("mouse_gastrulation", 2),
#'         rep("mouse_visual_cortex", 2)),
#'     version = rep(c("1.0.0", "2.0.0"), 2),
#'     ext_pattern = "\\.[Rr][Dd][Aa]$",
#'     doc_file = "inst/extdata/docuData/singlecellmultimodalv3.csv",
#'     pkg_name = "SingleCellMultiModal",
#'     dry.run = TRUE,
#' )
#'
#' make_metadata(
#'     directory = "~/data/scmm",
#'     dataDirs = "pbmc",
#'     version = "1.0.0",
#'     ext_pattern = "\\.[Rr][Dd][AaSs]$|\\.[Mm][Tt][Xx]\\.[Gg][Zz]$",
#'     doc_file = "inst/extdata/docuData/singlecellmultimodalv6.csv",
#'     pkg_name = "SingleCellMultiModal",
#'     dry.run = TRUE,
#' )
#'
#' @export
make_metadata <- function(
    directory = "~/data/scmm",
    dataDirs = c(rep("mouse_gastrulation", 2), rep("mouse_visual_cortex", 2), "pbmc"),
    version = c(rep(c("1.0.0", "2.0.0"), 2), "1.0.0"),
    ext_pattern = "\\.[Rr][Dd][AaSs]$|\\.[Mm][Tt][Xx]\\.[Gg][Zz]$|\\.[Hh]5$",
    doc_file,
    pkg_name = "SingleCellMultiModal",
    dry.run = TRUE,
    append = FALSE)
{
    if (!identical(basename(getwd()), pkg_name))
        stop("Run 'make_metadata()' from directory: ", pkg_name)

    exdata <- "inst/extdata"

    if (!dir.exists(exdata))
        dir.create(exdata)

    if (missing(doc_file))
        stop("'doc_file' for generating the metadata is missing")

    metafile <- file.path(exdata, "metadata.csv")

    metadat <- MetaHubCreate(
        base_dir = directory,
        data_dirs = dataDirs,
        ext_pattern = ext_pattern,
        doc_file = doc_file,
        version = version,
        pkg_name = pkg_name
    )

    if (!dry.run) {
        if(!append)
        {
            file.remove(metafile)
        }
        readr::write_csv(metadat, metafile, append = append, na="NA")
    }

    metadat
}

# make_metadata(
#     dataDirs = "mouse_gastrulation",
#     version = "1.0.0",
#     doc_file = "inst/extdata/docuData/singlecellmultimodalv1.csv",
#     dry_run = FALSE
# )
#
# make_metadata(
#     directory="CITEseq/",
#     dataDirs = "cord_blood",
#     version = "1.0.0",
#     doc_file = "inst/extdata/docuData/singlecellmultimodalv5.csv",
#     dry.run = FALSE,
#     append=TRUE
# )

# make_metadata(
#     dataDirs = c(rep("mouse_gastrulation", 2), "mouse_visual_cortex"),
#     version = c("1.0.0", "2.0.0", "1.0.0"),
#     doc_file = "inst/extdata/docuData/singlecellmultimodalv3.csv",
#     dry.run = FALSE
# )

# make_metadata(
#     directory = "~/data/scmm",
#     dataDirs = "peripheral_blood",
#     version = "1.0.0",
#     doc_file = "inst/extdata/docuData/singlecellmultimodalv5.csv",
#     dry.run = FALSE,
#     append = TRUE
# )

# make_metadata(
#     directory = "~/data/scmm",
#     dataDirs = "pbmc_10x",
#     version = "1.0.0",
#     doc_file = "inst/extdata/docuData/singlecellmultimodalv6.csv",
#     dry.run = FALSE,
#     append = TRUE
# )

# make_metadata(
#     directory = "../.localdata/SingleCellMultiModal/",
#     dataDirs = "macrophage_differentiation",
#     version = "1.0.0",
#     doc_file = "inst/extdata/docuData/singlecellmultimodalv7.csv",
#     dry.run = FALSE,
#     append = TRUE
# )

## request to update Maintainer field in older AH resources
# aq <- AnnotationHub::query(eh, "SingleCellMultiModal")
# aq[aq$maintainer == "Marcel Ramos <marcel.ramos@roswellpark.org>" &
#     grepl("v[12]", aq$rdatapath)]

# make_metadata(
#     directory = "~/data/scmm",
#     dataDirs = "mouse_embryo_8_cell",
#     version = "1.0.0",
#     doc_file = "inst/extdata/docuData/singlecellmultimodalv8.csv",
#     dry.run = FALSE,
#     append = TRUE
# )

make_metadata(
    directory = "~/data/scmm",
    dataDirs = "pbmc_10x",
    version = "1.0.1",
    doc_file = "inst/extdata/docuData/singlecellmultimodalv9.csv",
    dry.run = FALSE,
    append = TRUE
)

## Check metadata.csv file with:
ExperimentHubData::makeExperimentHubMetadata(
    file.path(Sys.getenv("HOME"), "gh/SingleCellMultiModal"), "metadata.csv"
)

# get data from cloudstor
# https://cloudstor.aarnet.edu.au/plus/s/Xzf5vCgAEUVgbfQ/download?path=%2Foutput&files=scnmtseq_gastrulation_mae_826-cells_orderedFeatures.rds
## ./output/scnmtseq_gastrulation_mae_826-cells_orderedFeatures.rds
library(MultiAssayExperiment)

ddir <- "~/data/scmm/mouse_gastrulation"

if (!dir.exists(ddir))
    dir.create(ddir, recursive = TRUE)

#   old
#   "scnmtseq_gastrulation_mae_826-cells_orderedFeatures.rds"
scnmt <- readRDS(
    file.path(ddir, "allcells",
        "scnmtseq_gastrulation_mae_AllCells.rds"
    )
)

exportClass(scnmt, ddir, fmt = "csv")

# convert .csv files to .rda matrices
.convertData <- function(
    directory = "~/data/scmm/",
    dataDir = "mouse_gastrulation",
    version = "1.0.0",
    pattern = ".csv")
{
    location <- file.path(directory, dataDir, paste0("v", version))
    csvs <- list.files(location, pattern = pattern, full.names = TRUE,
        recursive = FALSE)
    invisible(
        lapply(csvs, function(csvfile) {
            objname <- gsub(pattern, "", basename(csvfile))
            readin <- as.data.frame(readr::read_csv(csvfile))
            rnames <- readin[[1L]]

            if (!objname %in% c("scnmt_colData", "scnmt_sampleMap"))
                readin <- data.matrix(readin[, -1])
            else if (identical(objname, "scnmt_colData"))
                names(readin)[1] <- "cellID"
            else
                readin <- readin[, -1]

            if (!objname %in% "scnmt_sampleMap")
                rownames(readin) <- rnames

            assign(objname, readin)
            rdafile <- gsub("csv", "rda", csvfile)
            save(list = objname, file = rdafile)
        })
    )
}

.convertData()

# upload files to AWS S3
allextpat <- "\\.[Rr][Dd][AaSs]$|\\.[Mm][Tt][Xx]\\.[Gg][Zz]$|\\.[Hh]5$"

.version_folder <- function(version) {
        paste0("v", version)
}

.getDataFiles <- function(directory = "~/data/scmm",
    dataDir = "mouse_gastrulation", pattern = allextpat, version = "1.0.0"
) {
    vfolder <- .version_folder(version)
    location <- file.path(directory, dataDir, vfolder)
    list.files(
        location, pattern = pattern, full.names = TRUE, recursive = FALSE
    )
}

## check files are listed
.getDataFiles(dataDir = "pbmc_10x", version = "1.0.1")

# IMPORTANT!
# Make sure that AWS_DEFAULT_REGION, AWS_ACCESS_KEY_ID, and
# AWS_SECRET_ACCESS_KEY are set in the ~/.Renviron file

# source("inst/scripts/make-metadata.R")

upload_aws <- function(
    DataType, directory = "~/data/scmm", upload = FALSE,
    fileExt = allextpat, version = "1.0.0"
) {
    if (missing(DataType))
        stop("Enter a 'DataType' folder")
    datafilepaths <- .getDataFiles(
        directory = directory, dataDir = DataType,
        pattern = fileExt, version = version
    )
    vfolder <- .version_folder(version)
    bucketLocation <-
        file.path("experimenthub", "SingleCellMultiModal", DataType, vfolder)
    if (!upload)
        message("Data NOT uploaded")
    if (upload)
        AnnotationHubData:::upload_to_S3(file = datafilepaths,
            remotename = basename(datafilepaths),
            bucket = bucketLocation)
    else
        file.path("s3:/", bucketLocation, basename(datafilepaths))
}

# upload_aws(DataType = "mouse_gastrulation", version = "1.0.0", upload=TRUE)
# upload_aws(DataType = "mouse_gastrulation", version = "2.0.0", upload=TRUE)
# upload_aws(DataType = "cord_blood", directory="CITEseq", version = "1.0.0",
#     upload=TRUE)
# upload_aws(DataType = "mouse_visual_cortex", upload=TRUE)
# upload_aws(DataType = "pbmc_10x", directory = "~/data/scmm",
#     version = "1.0.0", upload = TRUE)
# upload_aws(DataType = "mouse_embryo_8_cell", directory = "~/data/scmm",
#     version = "1.0.0", upload = TRUE)
# upload_aws(DataType = "pbmc_10x", directory = "~/data/scmm",
#     version = "1.0.1", upload = TRUE)

#' Single-cell RNA sequencing and proteomics
#'
#' @description SCoPE2 assembles data on-the-fly from `ExperimentHub` to provide
#'   a
#'   [`MultiAssayExperiment`][MultiAssayExperiment::MultiAssayExperiment-class]
#'   container. The `DataType` argument provides access to the `SCoPE2` dataset
#'   as provided by Specht et al. (2020; DOI:
#'   <http://dx.doi.org/10.1101/665307>). The article provides more information
#'   about the data acquisition and pre-processing.
#'
#' @details The SCoPE2 study combines scRNA-seq (transcriptome) and
#'   single-cell proteomics.
#'
#'   * macrophage_differentiation: the cells are monocytes that undergo
#'   macrophage differentiation. No annotation is available for the
#'   transcriptome data, but batch and cell type annotations are
#'   available for the proteomics data in the `celltype` `colData` column.
#'   The transcriptomics and proteomics data were not measured from the same
#'   cells but from a distinct set of cell cultures.
#'   This dataset provides already filtered bad quality cells.
#'       * scRNAseq1 - single-cell transcriptome (batch 1)
#'       * scRNAseq2 - single-cell transcriptome (batch 2)
#'       * scp - single-cell proteomics
#'
#' @inheritParams scNMT
#'
#' @param DataType `character(1)` Indicates study that produces this type of
#'   data (default: 'macrophage_differentiation')
#'
#' @param modes `character()` A wildcard / glob pattern of modes, such as
#'   `"rna"`. A wildcard of `"*"` will return all modes, that are
#'   transcriptome ("rna") or proteome ("protein") which is the
#'   default.
#'
#' @param version `character(1)`, currently only version '1.0.0' is
#'   available
#'
#' @return A single cell multi-modal
#'   [`MultiAssayExperiment`][MultiAssayExperiment::MultiAssayExperiment-class]
#'   or informative `data.frame` when `dry.run` is `TRUE`
#'
#' @seealso SingleCellMultiModal-package
#'
#' @source All files are linked from the slavovlab website
#'     <https://scope2.slavovlab.net/docs/data>
#'
#' @references
#'   Specht, Harrison, Edward Emmott, Aleksandra A. Petelski, R.
#'   Gray Huffman, David H. Perlman, Marco Serra, Peter Kharchenko,
#'   Antonius Koller, and Nikolai Slavov. 2020. “Single-Cell
#'   Proteomic and Transcriptomic Analysis of Macrophage
#'   Heterogeneity.” bioRxiv. https://doi.org/10.1101/665307.
#'
#' @examples
#'
#' SCoPE2(DataType = "macrophage_differentiation",
#'        modes = "*",
#'        version = "1.0.0",
#'        dry.run = TRUE)
#'
#' @export
SCoPE2 <- function(
    DataType = "macrophage_differentiation",
    modes = "*",
    version = "1.0.0",
    dry.run = TRUE,
    verbose = TRUE,
    ...
) {
    if (version != "1.0.0")
        stop("Only version '1.0.0' is available.")

    ## Retrieve the different resources from ExperimentHub
    ess_list <- .getResourcesList(
        prefix = "macrophage_",
        datatype = DataType,
        modes = modes,
        version = version,
        dry.run = dry.run,
        verbose = verbose,
        ...
    )
    ## If dry.run, return only the information table
    if (dry.run) return(ess_list)
    ## Get the colData
    cd <- .mergeLowColData(ess_list[["experiments"]])
    colnames(cd)[which(colnames(cd) == "Batch")] <- "batch_Chromium"

    ## Construct and return the MAE object
    MultiAssayExperiment(
        experiments = ess_list[["experiments"]],
        colData = cd
    )
}

## read in
onto <- readr::read_tsv("inst/extdata/ontomap.tsv")

## modification
onto <- as.data.frame(onto)
onto[onto$DataType == "macrophage_differentiation_protein", "DataType"] <-
    "macrophage_differentiation"

## output checking
stopifnot(
    identical(length(unique(onto[["DataType"]])), 4L)
)

## writing
write.table(
    x = onto, file = "inst/extdata/ontomap.tsv",
    quote = FALSE, sep = "\t", row.names = FALSE
)

## reading ontology terms from kelly ontomap based on an ontomap old version
cellontokelly <- as.data.frame(
    readr::read_tsv("~/Downloads/Cell type ontology - Sheet2.tsv")
)
onto <- as.data.frame(
    readr::read_tsv("inst/extdata/ontomap.tsv")
)

## removing repetitive rows for seqFISH/scRNAseq celltypes
## aligning with newer version of ontomap
cellontokelly <-
    cellontokelly[!cellontokelly$dataset_name=="mouse_visual_cortex_scRNAseq",]
ontokey <- paste0(onto$DataType,"_",onto$function_name)
ontokey <-
    gsub("SCoPE2", "protein_SCoPE2", gsub("scMultiome", "multiome", ontokey))
ontokey <- paste0(ontokey, "_", onto$original_cell_name)
kellykey <-
    paste0(cellontokelly$dataset_name, "_", cellontokelly$original_cell_name)
## reordering
cellontokelly <- cellontokelly[match(ontokey, kellykey),]

onto$ontology_ID <- cellontokelly$ontology_ID
onto$ontology_cell_name <- cellontokelly$ontology_cell_name

## writing
write.table(
    x = onto, file = "inst/extdata/ontomap.tsv",
    quote = FALSE, sep = "\t", row.names = FALSE
)


## adding celltypes for cord_blood citeseq
## 
load("cord_blood/v1.0.0/coldata_scRNAseq.rda")
cd <- coldata_scRNAseq
ct <- unique(cd$celltype)
ct <- ct[-which(is.na(ct))]

onto <- as.data.frame(
    readr::read_tsv("inst/extdata/ontomap.tsv")
)

cn <- colnames(onto)
ctcb <- data.frame("cord_blood", "CITEseq", "celltype", ct, NA, NA)
colnames(ctcb) <- cn
onton <- rbind.data.frame(onto, ctcb) 
write.table(
    x = onton, file = "inst/extdata/ontomap.tsv",
    quote = FALSE, sep = "\t", row.names = FALSE
)

## ----eval=FALSE---------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("SingleCellMultiModal")

## ----include=TRUE,results="hide",message=FALSE,warning=FALSE------------------
library(SingleCellMultiModal)
library(MultiAssayExperiment)

## ----echo=FALSE---------------------------------------------------------------
imgurl <- paste0(
    "https://github.com/waldronlab/MultiAssayExperiment/blob/",
    "c3c59a094e5a08111ee98b9f69579db5634d9fd4/vignettes/",
    "MultiAssayExperiment.png?raw=true"
)
knitr::include_graphics(
    path = imgurl
)

############################################################
#
# author: Ludwig Geistlinger
# date: 2021-03-24 18:17:27
#
# descr: G&T-seq data retrieval
#
############################################################

#' Parallel sequencing data of single-cell genomes and transcriptomes
#'
#' @description GTseq assembles data on-the-fly from `ExperimentHub` to provide
#'   a
#'   [`MultiAssayExperiment`][MultiAssayExperiment::MultiAssayExperiment-class]
#'   container. The `DataType` argument provides access to the
#'   `mouse_embryo_8_cell` dataset as obtained from Macaulay et al. (2015).
#'   Protocol information for this dataset is available from Macaulay et al.
#'   (2016). See references.
#'
#' @details G&T-seq is a combination of Picoplex amplified gDNA sequencing
#'   (genome) and SMARTSeq2 amplified cDNA sequencing (transcriptome) of the
#'   same cell. For more information, see Macaulay et al. (2015).
#'     * mouse_embryo_8_cell:
#'     this dataset was filtered for bad cells as specified in Macaulay
#'     et al. (2015).
#'         * genomic - integer copy numbers as detected from scDNA-seq
#'         * transcriptomic - raw read counts as quantified from scRNA-seq
#'
#' @section metadata:
#'   The `MultiAssayExperiment` metadata includes the original function call
#'   that saves the function call and the data version requested.
#'
#' @param DataType `character(1)` Indicates study that produces this type of
#'   data (default: 'mouse_embryo_8_cell')
#'
#' @param modes `character()` A wildcard / glob pattern of modes, such as
#'   `"*omic"`. A wildcard of `"*"` will return all modes including
#'   copy numbers ("genomic") and RNA-seq read counts ("transcriptomic"),
#'   which is the default.
#'
#' @param version `character(1)` Currently, only version '1.0.0'.
#'
#' @param dry.run `logical(1)` Whether to return the dataset names before actual
#'   download (default `TRUE`)
#'
#' @param verbose `logical(1)` Whether to show the dataset currently being
#'   (down)loaded (default `TRUE`)
#'
#' @param ... Additional arguments passed on to the
#'   [ExperimentHub][ExperimentHub::ExperimentHub-class] constructor
#'
#' @seealso SingleCellMultiModal-package
#'
#' @return A single cell multi-modal
#'   [MultiAssayExperiment][MultiAssayExperiment::MultiAssayExperiment-class] or
#'   informative `data.frame` when `dry.run` is `TRUE`
#'
#' @source <https://www.ebi.ac.uk/ena/browser/view/PRJEB9051>
#'
#' @references
#'   Macaulay et al. (2015) G&T-seq: parallel sequencing of single-cell
#'   genomes and transcriptomes. Nat Methods, 12:519–22.
#'
#'   Macaulay et al. (2016) Separation and parallel sequencing of the genomes
#'   and transcriptomes of single cells using G&T-seq. Nat Protoc, 11:2081–103.
#'
#' @examples
#'
#' GTseq()
#'
#' @export GTseq
GTseq <-
    function(
        DataType = "mouse_embryo_8_cell", modes = "*",
        version = "1.0.0", dry.run = TRUE, verbose = TRUE, ...
    )
{
    stopifnot(.isSingleChar(version), .isSingleChar(DataType))
    meta <- list(call = match.call())

    ess_list <- .getResourcesList(
        prefix = "GTseq_",
        datatype = DataType,
        modes = modes,
        version = version,
        dry.run = dry.run,
        verbose = verbose,
        ...
    )

    if (dry.run) { return(ess_list) }

    cdat <- ess_list[["colData"]]
    prim.ids <- rep(paste0("cell", seq_len(112)), 2)
    smap <- S4Vectors::DataFrame(
        assay = tolower(cdat[,"Comment.LIBRARY_SOURCE."]),
        primary = prim.ids,
        colname = cdat[,"Sample.ID"]
    )

    rcols <- c("organism", "sex", "cell.type")
    rcols <- paste0("Characteristics.", rcols, ".")
    cdat <- cdat[seq_len(112), rcols]
    rownames(cdat) <- prim.ids[seq_len(112)]

    MultiAssayExperiment(
        experiments = ess_list[["experiments"]],
        colData = cdat,
        sampleMap = smap,
        metadata = c(meta, as.list(ess_list[["metadata"]]))
    )
}

#' addCTLabels
#'
#' @param cd the `colData` `DataFrame`
#' @param out list data structure returned by `getCellGroups`
#' @param outname character indicating the name of the out data structure
#' @param ct character indicating the celltype to assign in the `ctcol`
#' @param mkrcol character indicating the cd column to store the markers
#' indicated by `outname` (default is markers)
#' @param ctcol character indicating the column in cd to store the cell type
#' indicated by `ct` (default is celltype)
#' @param overwrite logical indicating if the cell types have to be overwritten
#' without checking if detected barcodes were already assigned to other celltypes
#' @param verbose logical for having informative messages during the execution
#'
#' @return an updated version of the cd DataFrame
#'
#' @export
addCTLabels <- function(cd, out, outname, ct, mkrcol="markers", ctcol="celltype",
                        overwrite=FALSE, verbose=TRUE)
{
    ## adds to input cd colData in the mkrcol the markers indicated by outname
    ## and in the ctcol the celltype indicated in ct
    ## the positions for the barcodes (rows in the cd) are taken in position
    ## outname from the out structure given by function getCellGroups
    stopifnot(any(c(mkrcol, ctcol) %in% colnames(cd)))
    stopifnot((outname %in% names(out)))

    cellbc <- out[[outname]]$bc
    idxc <- which(rownames(cd) %in% cellbc)
    if (length(idxc) !=0)
    {
        if (overwrite)
        {
            if(verbose) message("Blindly overwriting cell types assignements")
            cd[[mkrcol]][idxc] <- outname
            cd[[ctcol]][idxc] <- ct
        } else {
            ## checking if celltypes are already assigned
            idxnona <- which(!is.na(cd[[mkrcol]][idxc]))
            # don't get why ifelse doesn't work
            # idxcnona <- ifelse(length(idxnona)!=0, idxc[-idxnona], idxc)
            if ( length(idxnona)!=0 ) {
                idxcnona <- idxc[-idxnona]
                if(verbose) message(length(idxnona), " Barcodes already assigned.\n",
                                    "Assigning only ", length(idxcnona), " Barcodes...")
            } else { idxcnona <- idxc }
            if (length(idxcnona)!=0)
            {
                cd[[mkrcol]][idxcnona] <- outname
                cd[[ctcol]][idxcnona] <- ct
            } else {
                if(verbose) message("All selected Barcodes are already assigned\n",
                                    "Look at the overwrite argument to handle a more ",
                                    "brutal behaviour")
            }
        }

    } else {
        warning("No barcodes in cd detected for the selected ", outname,
                "\nReturning cd as it is...")
    }

    return(cd)
}

#' @importFrom graphics abline smoothScatter
.plotGatingAdt <- function(mat, adt1="CD19", adt2="CD3", th1=0.2, th2=0)
{
    plot(x=mat[adt1,], y=mat[adt2,], xlab=adt1, ylab=adt2,
         main=paste0("Gain plot with x-th: ", th1, " y-th: ", th2))
    abline(v=th1, col="red", lty=2)
    abline(h=th2, col="red", lty=2)
    smoothScatter(x=mat[adt1,], y=mat[adt2,], xlab=adt1, ylab=adt2,
                  main=paste0("Gain plot with x-th: ", th1, " y-th: ", th2))

    abline(v=th1, col="red", lty=2)
    abline(h=th2, col="red", lty=2)
}


#' getCellGroups
#'
#' @description
#' Shows the cells/barcodes in two different plots (scatter and density)
#' divinding the space in four quadrant indicated by the two thresholds given
#' as input parameters.
#' The x/y-axis represent respectively the two ADTs given as input.
#' It returns a list of one element for each quadrant, each with barcodes and
#' percentage (see Value section for details).
#'
#' @param mat matrix of counts or clr transformed counts for ADT data in CITEseq
#' @param adt1 character indicating the name of the marker to plot on the x-axis
#' (default is CD19).
#' @param adt2 character indicating the name of the marker to plot on the y-axis
#' (default is CD3).
#' @param th1 numeric indicating the threshold for the marker on the x-axis
#' (default is 0.2).
#' @param th2 numeric indicating the threshold for the marker on the y-axis
#' (default is 0).
#'
#' @return a list of four different element, each one indicating the quarter
#' where the thresholds divide the plotting space, in eucledian order I, II,
#' III, IV quadrant, indicating respectively +/+, +/-, -/+, -/- combinations
#' for the couples of selected ADTs.
#' Each element of the list contains two objects, one with the list of detected
#' barcodes and one indicating the percentage of barcodes falling into that
#' quadrant.
#' .
#' @details helps to do manual gating for cell type indentification with CITEseq
#' or similar data, providing cell markers.
#' Once identified two interesting markers for a cell type, the user has to
#' play with the thresholds to identify the cell populations specified by an
#' uptake (+) o downtake (-) of the couple of markers (ADTs) previously selected.
#'
#' @importFrom graphics text
#'
#' @export
getCellGroups <- function(mat, adt1="CD19", adt2="CD3", th1=0.2, th2=0)
{
    stopifnot(any(adt1,adt2) %in% rownames(mat))

    plot <- match.arg(plot)
    .plotGatingAdt(mat, adt1, adt2, th1, th2)
    matadt <- mat[c(adt1,adt2),]
    adt1p <- (matadt[adt1,]>th1)
    adt1m <- (matadt[adt1,]<=th1)
    adt2p <- (matadt[adt2,]>th2)
    adt2m <- (matadt[adt2,]<=th2)


    if (sum(adt1p)+sum(adt1m) != dim(mat)[2]) stop("something went wrong with adt1")
    if (sum(adt2p)+sum(adt2m) != dim(mat)[2]) stop("something went wrong with adt2")

    adt12pp <- which(adt1p & adt2p)
    adt12pm <- which(adt1p & adt2m)
    adt12mp <- which(adt1m & adt2p)
    adt12mm <- which(adt1m & adt2m)

    l <- list(
        ADT12pp=list(
            bc=colnames(matadt)[adt12pp],
            prc=((length(adt12pp)/dim(matadt)[2])*100)),
        ADT12pm=list(
            bc=colnames(matadt)[adt12pm],
            prc=((length(adt12pm)/dim(matadt)[2])*100)),
        ADT12mp=list(
            bc=colnames(matadt)[adt12mp],
            prc=((length(adt12mp)/dim(matadt)[2])*100)),
        ADT12mm=list(
            bc=colnames(matadt)[adt12mm],
            prc=((length(adt12mm)/dim(matadt)[2])*100))
    )
    names(l) <- c(paste0(adt1,"+/",adt2,"+"),
                  paste0(adt1,"+/",adt2,"-"),
                  paste0(adt1,"-/",adt2,"+"),
                  paste0(adt1,"-/",adt2,"-"))


    text((min(matadt[adt1,])+0.03), (max(matadt[adt2,])-0.05), paste(round(l[[3]]$prc), "%"))
    text((max(matadt[adt1,])-0.03), (max(matadt[adt2,])-0.05), paste(round(l[[1]]$prc), "%"))
    text((max(matadt[adt1,])-0.03), (min(matadt[adt2,])+0.05), paste(round(l[[2]]$prc), "%"))
    text((min(matadt[adt1,])+0.03), (min(matadt[adt2,])+0.05), paste(round(l[[4]]$prc), "%"))
    return(l)
}

## Load HDF5 file with either TENxMatrix or HDF5Array
.getH5_TENx <- function(filelist, ehub, fn, verbose) {
    if (verbose)
        message("Working on: ", paste(fn, collapse = ",\n "))
    se_h5 <- grep("_se", filelist, value = TRUE)
    se_obj <- query(ehub, se_h5)[[1L]]

    hasTENx <- any(grepl("tenx", filelist))
    patt <- if (hasTENx) "tenx" else "_assay"

    h5data <- grep(patt, filelist, value = TRUE, ignore.case = TRUE)
    h5fileloc <- query(ehub, h5data)[[1L]]

    if (!hasTENx)
        h5array <- HDF5Array::HDF5Array(h5fileloc, "assay001", as.sparse = TRUE)
    else
        h5array <- HDF5Array::TENxMatrix(h5fileloc, "pbmc")

    SummarizedExperiment::`assays<-`(
        x = se_obj, withDimnames = FALSE,
        value = list(counts = h5array)
    )
}

.loadHDF5 <- function(ehub, filepaths, verbose) {
    matchres <- grepl("\\.[Hh]5|_se\\.[Rr][Dd][Ss]", filepaths)
    fpaths <- filepaths[matchres]
    fact <- .removeExt(fpaths)
    fact <- gsub("_se|_assays|_tenx", "", fact)
    h5list <- split(fpaths, fact)
    lapply(h5list,
        .getH5_TENx,
        ehub = ehub, fn = names(h5list), verbose = verbose
    )
}

.message <-
    function(...)
{
    message(...)
    TRUE
}

## @mtmorgan's function from HCAMatrixBrowser
.read_mtx <-
    function(path, verbose = FALSE)
{
    headers <- readLines(path, 2L)
    dims <- as.integer(strsplit(headers[2], " ")[[1]][c(1, 2)])
    !verbose || .message("dim: ", dims[1], " ", dims[2])
    v <- scan(
        path, list(integer(), integer(), numeric()), skip = 2,
        quiet = !verbose
    )
    Matrix::sparseMatrix(v[[1]], v[[2]], x = v[[3]], dims = dims)
}

.loadMTX <- function(ehub, filepaths, verbose) {
    matchres <-
        grepl("\\.[Mm][Tt][Xx]\\.[Gg][Zz]$|_se\\.[Rr][Dd][Ss]$", filepaths)
    filepaths <- filepaths[matchres]
    fact <- .removeExt(filepaths)
    fact <- gsub("_se", "", fact)
    mtxlist <- split(filepaths, fact)
    lapply(mtxlist, function(mtxfile, fn) {
        if (verbose)
            message("Working on: ", paste(fn, collapse = ",\n "))
        se_mtx <- grep("_se", mtxfile, value = TRUE)
        mtxdata <- grep("mtx", mtxfile, value = TRUE, ignore.case = TRUE)
        se <- query(ehub, se_mtx)[[1L]]
        mtxfile <- query(ehub, mtxdata)[[1L]]
        mtxf <- .read_mtx(mtxfile)

        BiocBaseUtils::setSlots(
            object = se,
            assays = SummarizedExperiment::Assays(
                S4Vectors::SimpleList(counts = mtxf)
            )
        )
    }, fn = names(mtxlist))
}

#' Single-cell Multiome ATAC + Gene Expression
#'
#' @description 10x Genomics Multiome technology enables simultaneous profiling
#' of the transcriptome (using 3’ gene expression) and epigenome
#' (using ATAC-seq) from single cells to
#' deepen our understanding of how genes are expressed and regulated across
#' different cell types. Data prepared by Ricard Argelaguet.
#'
#' @details Users are able to choose from either an `MTX` or `HDF5` file format
#'   as the internal data representation. The `MTX` (Matrix Market) format
#'   allows users to load a sparse `dgCMatrix` representation. Choosing `HDF5`
#'   gives users a sparse `HDF5Array` class object.
#'     * pbmc_10x: 10K Peripheral Blood Mononuclear Cells provided by
#' [10x Genomics website](https://support.10xgenomics.com/single-cell-multiome-atac-gex/datasets)
#'     Cell quality control filters are available in the object `colData`
#'     together with the `celltype` annotation labels.
#'
#' @inheritParams scNMT
#'
#' @param format `character(1)` Either MTX or HDF5 data format (default MTX)
#'
#' @return A 10X PBMC `MultiAssayExperiment` object
#'
#' @examples
#'
#' scMultiome(DataType = "pbmc_10x", modes = "*", dry.run = TRUE)
#'
#' @export
scMultiome <-
    function(
        DataType = "pbmc_10x", modes = "*", version = "1.0.0",
        format = c("MTX", "HDF5"), dry.run = TRUE, verbose = TRUE, ...
    )
{
    stopifnot(.isSingleChar(version), .isSingleChar(DataType))

    format <- match.arg(format)
    meta <- list(call = match.call(), version = version)

    if (!version %in% c("1.0.0", "1.0.1"))
        stop("Invalid 'version'; see '?scMultiome' for details.")

    ess_list <- .getResourcesList(prefix = "pbmc_", datatype = DataType,
        modes = modes, version = version, dry.run = dry.run,
        verbose = verbose, format = format, ...)

    if (dry.run) { return(ess_list) }

    MultiAssayExperiment(
        experiments = ess_list[["experiments"]],
        colData = ess_list[["colData"]],
        sampleMap = ess_list[["sampleMap"]],
        metadata = meta
    )
}

#' @importFrom ExperimentHub loadResources ExperimentHub
#' @importFrom AnnotationHub query
#' @importFrom utils glob2rx read.csv
#' @import MultiAssayExperiment
NULL

#' SingleCellMultiModal-package
#'
#' @aliases NULL SingleCellMultiModal-package
#'
#' @description
#' The SingleCellMultiModal package provides a convenient and user-friendly
#' representation of multi-modal data from project such as `scNMT` for mouse
#' gastrulation.
#'
#' @examples
#' help(package = "SingleCellMultiModal")
#'
"_PACKAGE"

## Make the data to distribute to ExperimentHub

## Required packages
library(HDF5Array)
library(BiocFileCache)
library(SingleCellExperiment)

## Steps:
## 1. Retrieve the scRNASeq matrices (n=2) from NCBI
## 2. Read the count matrices
## 3. Combine the two RNA count matrices in a SingleCellExperiment
## 4. Retrieve the protein matrix and annotation from Google Drive
## 5. Combine the protein data in a SingleCellExperiment object

## Note that step 3 is optional. I needed to migrate to HDF5 due to 
## memory limitations.

## -------------------------------------- ##
## 1. Retrieve the scRNASeq matrices (n=2) from NCBI
## -------------------------------------- ##

## See also https://bioconductor.org/packages/devel/bioc/vignettes/MultiAssayExperiment/inst/doc/UsingHDF5Array.html
bfc <- BiocFileCache()
url1 <- "ftp://ftp.ncbi.nlm.nih.gov/geo/samples/GSM4226nnn/GSM4226877/suppl/GSM4226877_rna_data_Bio_Replicate_1.csv.gz"
url2 <- "ftp://ftp.ncbi.nlm.nih.gov/geo/samples/GSM4226nnn/GSM4226878/suppl/GSM4226878_rna_data_Bio_Replicate_2.csv.gz"
bfcrpath(bfc, url1)
bfcrpath(bfc, url2)

## ---------------------- ##
## 2. Read count matrices
## ---------------------- ##

## Batch 1
m1 <- data.table::fread(file = bfcquery(bfc, "GSM4226877")$rpath,
                        sep = ",", header = TRUE)
rn <- m1[[1]]
m1 <- as.matrix(m1[, -1])
rownames(m1) <- rn
colnames(m1) <- paste0(colnames(m1), ".1")
## Batch 2
m2 <- data.table::fread(file = bfcquery(bfc, "GSM4226878")$rpath,
                        sep = ",", header = TRUE)
rn <- m2[[1]]
m2 <- as.matrix(m2[, -1])
colnames(m2) <- paste0(colnames(m2), ".2")
rownames(m2) <- rn

## ------------------------------------------------------- ##
## 3. Combine the two RNA count matrices in a SingleCellExperiment
## ------------------------------------------------------- ##

m1 <- DelayedArray(m1)
m2 <- DelayedArray(m2)
m3 <- cbind(m1, m2) ## This process is delayed until writing
batch <- factor(gsub("^.*[.](\\d)$", "\\1", colnames(m3)))
sce <- SingleCellExperiment(list(counts = m3),
                            colData = DataFrame(Batch = batch))
## The object is rather big and is better stored on disk as an HDF5
saveHDF5SummarizedExperiment(sce, 
                             dir = "../.localdata/SingleCellMultiModal/macrophage_differentiation/v1.0.0/",
                             prefix = "macrophage_rna_", 
                             as.sparse = TRUE)
## Restore some RAM
rm(m1, m2, m3); gc()

## ------------------------------------------------------- ##
## 4. Retrieve the protein matrix and annotation from Google Drive
## ------------------------------------------------------- ##

## Download the protein data provided by the Slavov lab
## https://drive.google.com/file/d/1sF5STkofF_f2msnYaaYdWabou84Qf2Xr/view?usp=sharing
protein_assay <- read.csv("../.localdata/SCP/specht2019/v3/Proteins-processed.csv",
                          row.names = 1)
protein_assay <- protein_assay[, colnames(protein_assay) != "protein"]
protein_assay <- as.matrix(protein_assay)

## Download the protein data provided by the Slavov lab
## https://drive.google.com/file/d/16vf6rjIsk-oK9naAH6BQnCFrlWnYtJsS/view?usp=sharing
protein_colData <- read.csv("../.localdata/SCP/specht2019/v3/Cells.csv",
                            row.names = 1)
protein_colData <- t(protein_colData)
protein_colData <- DataFrame(protein_colData)
## Replace the cell type annotation by more explicit values
protein_colData$celltype <- 
    ifelse(protein_colData$celltype == "sc_m0", "Macrophage", "Monocyte")
## Rename the `raw.file` value by `Batch`
colnames(protein_colData)[5] <- "batch_MS"

## ------------------------------------------------------- ##
## 6. Combine the data in a SingleCellExperiment object
## ------------------------------------------------------- ##

macrophage_protein <- SingleCellExperiment(assay = list(logexprs = protein_assay),
                                           colData = protein_colData)
format(object.size(macrophage_protein), "MB")
## Note the protein data can easily fit in memory. We save it as an Rda
save(macrophage_protein, file = "../.localdata/SingleCellMultiModal/macrophage_differentiation/v1.0.0/macrophage_protein.Rda")

## ------------------------------------------------------- ##
## Conclusion
## ------------------------------------------------------- ##

## These files should be sent to ExperimentHub:
##  mRNA
##  - macrophage_rna_assays.h5
##  - macrophage_rna_se.rds
##  Protein
##  - macrophage_protein.Rda

# version 1
scmeta <- data.frame(
    DataProvider = "Dept. of Bioinformatics, The Babraham Institute, United Kingdom",
    TaxonomyId = "10090",
    Species = "Mus musculus",
    SourceUrl = "https://cloudstor.aarnet.edu.au/plus/s/Xzf5vCgAEUVgbfQ",
    SourceType = "RDS",
    SourceVersion = "1.0.0",
    DataType = "mouse_gastrulation",
    Maintainer  = "Ricard Argelaguet <ricard@ebi.ac.uk>",
    stringsAsFactors = FALSE
)
write.csv(
    scmeta,
    file = "inst/extdata/docuData/singlecellmultimodalv1.csv",
    row.names = FALSE
)

# version 2
scmeta <- data.frame(
    DataProvider =
        "Dept. of Bioinformatics, The Babraham Institute, United Kingdom",
    TaxonomyId = "10090",
    Species = "Mus musculus",
    SourceUrl = "https://cloudstor.aarnet.edu.au/plus/s/Xzf5vCgAEUVgbfQ",
    SourceType = "RDS",
    SourceVersion = c("1.0.0", "2.0.0"),
    DataType = "mouse_gastrulation",
    Maintainer  = "Ricard Argelaguet <ricard@ebi.ac.uk>",
    stringsAsFactors = FALSE
)
write.csv(
    scmeta,
    file = "inst/extdata/docuData/singlecellmultimodalv2.csv",
    row.names = FALSE
)

# version 3 with spatial
scmeta <- data.frame(
    DataProvider = c(
        rep("Dept. of Bioinformatics, The Babraham Institute, United Kingdom", 2),
        rep("Dept. of Molecular Genetics, Allen Institute for Brain Science, United States", 2)
    ),
    TaxonomyId = "10090",
    Species = "Mus musculus",
    SourceUrl = c(
        rep("https://cloudstor.aarnet.edu.au/plus/s/Xzf5vCgAEUVgbfQ", 2),
        "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE71585",
        "https://www.dropbox.com/sh/avj4nrd4la5i88u/AACafWwBbE-xsLvOGDwRZDpYa?dl=0"
    ),
    SourceType = c("RDS", "RDS", "TXT", "TXT"),
    SourceVersion = c("1.0.0", "2.0.0", "1.0.0", "2.0.0"),
    DataType = c(rep("mouse_gastrulation", 2), rep("mouse_visual_cortex", 2)),
    Maintainer = c(rep("Ricard Argelaguet <ricard@ebi.ac.uk>", 2),
                   rep("Dario Righelli <dario.righelli@gmail.com>", 2)),
    stringsAsFactors = FALSE
)
write.csv(
    scmeta,
    file = "inst/extdata/docuData/singlecellmultimodalv3.csv",
    row.names = FALSE
)


# version 4 with cord_blood
scmeta <- data.frame(
    DataProvider = c(
        rep("Dept. of Bioinformatics, The Babraham Institute, United Kingdom", 2),
        rep("Dept. of Molecular Genetics, Allen Institute for Brain Science, United States", 2),
        "Innovation Lab, New York Genome Center, New York, United States"
    ),
    TaxonomyId = c(rep("10090",4), "9606"),
    Species = c(rep("Mus musculus", 4), "Homo sapiens"),
    SourceUrl = c(
        rep("https://cloudstor.aarnet.edu.au/plus/s/Xzf5vCgAEUVgbfQ", 2),
        rep("https://www.dropbox.com/sh/avj4nrd4la5i88u/AACafWwBbE-xsLvOGDwRZDpYa?dl=0", 2),
        "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE100866"
    ),
    SourceType = c(rep("RDS", 2), rep("TXT",3)),
    SourceVersion = c("1.0.0", "2.0.0", "1.0.0", "2.0.0", "1.0.0"),
    DataType = c(rep("mouse_gastrulation", 2), rep("mouse_visual_cortex",2), "coord_blood"),
    Maintainer = c(rep("Ricard Argelaguet <ricard@ebi.ac.uk>", 2),
                   rep("Dario Righelli <dario.righelli@gmail.com>",3)),
    stringsAsFactors = FALSE
)

write.csv(
    scmeta,
    file = "inst/extdata/docuData/singlecellmultimodalv3.csv",
    row.names = FALSE
)

# indv cord_blood
citeseqmeta <- data.frame(
    DataProvider =
        "Innovation Lab, New York Genome Center, New York, United States",
    TaxonomyId = "9606",
    Species = "Homo sapiens",
    SourceUrl = "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE100866",
    SourceType = "TXT",
    SourceVersion = "1.0.0",
    DataType = "coord_blood",
    Maintainer = "Dario Righelli <dario.righelli@gmail.com>",
    stringsAsFactors = FALSE
)

write.csv(
    citeseqmeta,
    file = "inst/extdata/docuData/singlecellmultimodalv5.csv",
    row.names = FALSE
)
#
#
# # version 2 with spatial
# scmeta <- data.frame(
#     DataProvider = c(
#         rep("Dept. of Bioinformatics, The Babraham Institute, United Kingdom", 2),
#         rep("Dept. of Molecular Genetics, Allen Institute for Brain Science, United States", 2),
#         "Innovation Lab, New York Genome Center, New York, United States"
#     ),
#     TaxonomyId = c(rep("10090",4), "9606"),
#     Species = c(rep("Mus musculus", 4), "Homo sapiens"),
#     SourceUrl = c(
#         rep("https://cloudstor.aarnet.edu.au/plus/s/Xzf5vCgAEUVgbfQ", 2),
#         rep("https://www.dropbox.com/sh/avj4nrd4la5i88u/AACafWwBbE-xsLvOGDwRZDpYa?dl=0", 2),
#         "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE100866"
#     ),
#     SourceType = c(rep("RDS", 2), rep("TXT",3)),
#     SourceVersion = c("1.0.0", "2.0.0", "1.0.0", "2.0.0", "1.0.0"),
#     DataType = c(rep("mouse_gastrulation", 2), rep("mouse_visual_cortex",2), "coord_blood"),
#     Maintainer = c(rep("Marcel Ramos <marcel.ramos@roswellpark.org>", 2),
#                    rep("Dario Righelli <dario.righelli@gmail.com>",3)),
#     stringsAsFactors = FALSE
# )
# write.csv(
#     scmeta,
#     file = "inst/extdata/docuData/singlecellmultimodalv3.csv",
#     row.names = FALSE
# )

# version 5 pbmc
scmeta <- data.frame(
    DataProvider = "European Bioinformatics Institute (EMBL-EBI), United Kingdom",
    TaxonomyId = "9606",
    Species = "Homo sapiens",
    SourceUrl = "http://ftp.ebi.ac.uk/pub/databases/mofa/10x_rna_atac_vignette/filtered_feature_bc_matrix/",
    SourceVersion = "1.0.0",
    DataType = "pbmc_10x",
    Maintainer  = "Ricard Argelaguet <ricard@ebi.ac.uk>",
    stringsAsFactors = FALSE
)

write.csv(
    scmeta,
    file = "inst/extdata/docuData/singlecellmultimodalv6.csv",
    row.names = FALSE
)

## version 7: creating metadata for the SCoPE2 dataset
scope2meta <- data.frame(
    DataProvider = paste0("Slavov Laboratory and SCP Center at ",
                          "Northeastern University, Boston, United ",
                          "states"),
    TaxonomyId = "9606",
    Species = "Homo sapiens",
    SourceUrl = c("https://drive.google.com/file/d/1sF5STkofF_f2msnYaaYdWabou84Qf2Xr/view?usp=sharing",
                  "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE142392",
                  "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE142392"),
    SourceType = c("CSV", "CSV", "CSV"),
    SourceVersion = "1.0.0",
    DataType = "macrophage_differentiation",
    Maintainer = "Christophe Vanderaa <christophe.vanderaa@uclouvain.be>",
    stringsAsFactors = FALSE
)

write.csv(
    scope2meta,
    file = "inst/extdata/docuData/singlecellmultimodalv7.csv",
    row.names = FALSE
)

# version 8: GTseq dataset
gtseq <- data.frame(
    DataProvider = "Wellcome Trust Sanger Institute, Cambridge, United Kingdom",
    TaxonomyId = "10090",
    Species = "Mus musculus",
    SourceUrl = "https://www.ebi.ac.uk/ena/browser/view/PRJEB9051",
    SourceVersion = "1.0.0",
    DataType = "mouse_embryo_8_cell",
    Maintainer  = "Ludwig Geistlinger <ludwig_geistlinger@hms.harvard.edu>",
    stringsAsFactors = FALSE
)

write.csv(
    gtseq,
    file = "inst/extdata/docuData/singlecellmultimodalv8.csv",
    row.names = FALSE
)


scmeta9 <- data.frame(
    DataProvider = "European Bioinformatics Institute (EMBL-EBI), United Kingdom",
    TaxonomyId = "9606",
    Species = "Homo sapiens",
    SourceUrl = "http://ftp.ebi.ac.uk/pub/databases/mofa/10x_rna_atac_vignette/filtered_feature_bc_matrix/",
    SourceVersion = "1.0.1",
    DataType = "pbmc_10x",
    Maintainer  = "Marcel Ramos <marcel.ramos@roswellpark.org>",
    stringsAsFactors = FALSE
)

write.csv(
    scmeta9,
    file = "inst/extdata/docuData/singlecellmultimodalv9.csv",
    row.names = FALSE
)

library(devtools)
load_all()
mae <- CITEseq("cord_blood", dry.run=FALSE)

## Detecting MOUSE/HUMAN cells
rna <- assays(mae)[["scRNAseq"]]
hrna <- colSums(rna[grep("^HUMAN", rownames(rna)),])
mrna <- colSums(rna[grep("^MOUSE", rownames(rna)),])
mate <- cbind(hrna, mrna)
plot(log1p(hrna+1), log1p(mrna+1), xlab="hrna", ylab="mrna")
## Using kmeans for detecting 2 bigger clusters of human and mouse cells
set.seed(666)
km <- kmeans(cbind(log(hrna+1), log(mrna+1)), centers=2)
plot(log(hrna+1), log(mrna+1), xlab="hrna", ylab="mrna", col=km$cluster)

## computing distance+hclust on human/mouse cells cluster for detecting mixed cells
mat <- cbind(log(hrna+1), log(mrna+1))

## human cells
d <- dist(mat[km$cluster==2,])
hc <- hclust(d, method="single")
cl <- cutree(hc, k=2)
plot(mat[km$cluster==2,], col=cl)
hbc <- names(cl)[cl==1]

# cd <- colData(mae)
load("cord_blood/v1.0.0/coldata_scRNAseq.rda")
cd <- coldata_scRNAseq

cd$species <- NA
cd$species[which(rownames(cd) %in% hbc)] <- "HUMAN"
cd$species[which(rownames(cd) %in% names(cl)[cl==2])] <- "MIXED"
cd$species[which(rownames(cd) %in% names(km$cluster)[km$cluster==1])] <- "MOUSE"
table(cd$specie)

##### Annotating cell types
adtclrgeo <- as.matrix(read.csv("~/Downloads/GSE100866_CBMC_8K_13AB_10X-ADT_clr-transformed.csv", row.names=1))
## add this assay to the ADTs 
cd$celltype <- NA
cd$markers <- NA

cdct <- cd[!cd$discard,]
cdct <- cdct[cdct$species=="HUMAN",]

out.cd19.cd3 <- getCellGroups(adtclrgeo, adt1="CD19", adt2="CD3", th1=0.9, th2=0.6)


cdct <- addCTLabels(cdct, out.cd19.cd3, "CD19-/CD3+", "T-cells")
cdct <- addCTLabels(cdct, out.cd19.cd3, "CD19+/CD3-", "B-cells")

out.cd11.cd14 <- getCellGroups(adtclrgeo, adt1="CD11c", adt2="CD14", th1=0.4, th2=0.55)
cdct <- addCTLabels(cdct, out.cd11.cd14, "CD11c+/CD14+", "Monocytes CD14+")
table(cdct$celltype)

out.cd11.cd16 <- getCellGroups(adtclrgeo, adt1="CD11c", adt2="CD16", th1=0.4, th2=0.55)
cdct <- addCTLabels(cdct, out.cd11.cd16, "CD11c+/CD16+", "Monocytes CD16+")
table(cdct$celltype)

out.T.cd8.cd4 <- getCellGroups(adtclrgeo[,out.cd19.cd3$`CD19-/CD3+`$bc], adt1="CD8", adt2="CD4", th1=0.9, th2=0.6)
## overwriting because CD4/CD8 T-cells are subgroups of T-cells
cdct <- addCTLabels(cdct, out.T.cd8.cd4, "CD8-/CD4+", "CD4 T-cells", overwrite=TRUE)
cdct <- addCTLabels(cdct, out.T.cd8.cd4, "CD8+/CD4-", "CD8 T-cells", overwrite=TRUE)
# cord_blood_colData_anno <- cdct
# save(cord_blood_colData_anno, file="cord_blood_colData_anno.rda")

## precursors are CD34+ and I took CD56- which seems not expressed from the paper figure
out.cd56.cd34 <- getCellGroups(adtclrgeo, adt1="CD56", adt2="CD34", th1=0.37, th2=0.9)
prebc <- out.cd56.cd34$`CD56-/CD34+`$bc
# idxpre <- which(rownames(cdct) %in% prebc)
# which(prebc %in% rownames(cdct)[!is.na(cdct$celltype)]) ## showing overlap!!!
cdct <- addCTLabels(cdct, out.cd56.cd34, "CD56-/CD34+", "Precursors")

# ## NATURAL KILLERS are CD3-/CD16+ (CD56+ and CD16+)
out.cd16.cd3 <- getCellGroups(adtclrgeo, adt1="CD16", adt2="CD3", th1=0, th2=0.55)
nkcellbc16 <- out.cd16.cd3$`CD16+/CD3-`$bc
idxnk <- which(rownames(cdct) %in% nkcellbc16)
length(idxnk)
sum(nkcellbc16 %in% rownames(cdct)[!is.na(cdct$celltype)]) ## showing overlap!!!

cdctnk <- addCTLabels(cdct, out.cd16.cd3, "CD16+/CD3-", "Natural Killers")

## other markers for NK are CD56+ and CD3-
out.cd56.cd3 <- getCellGroups(adtclrgeo, adt1="CD56", adt2="CD3", th1=0, th2=0)
# nkcellbc56 <- out.cd56.cd3$`CD56+/CD3-`$bc
# idxnk <- which(rownames(cdct) %in% nkcellbc56)
# length(idxnk)
# sum(nkcellbc56 %in% rownames(cdct)[!is.na(cdct$celltype)]) ## showing overlap!!!
cdctnk <- addCTLabels(cdctnk, out.cd56.cd3, "CD56+/CD3-", "Natural Killers")

coldata_scRNAseq <-  cdctnk
save(coldata_scRNAseq, file="cord_blood/v1.0.0/coldata_scRNAseq.rda")
scADT_clrCounts <- adtclrgeo
save(scADT_clrCounts, file="cord_blood/v1.0.0/scADT_clrCounts.rda")


## Building tsne
cnts <- (mae[["scADT"]][, which(colnames(mae[["scADT"]]) %in% rownames(cdctnk))])
adtclrgeoss <- adtclrgeo[,which(colnames(adtclrgeo) %in% rownames(cdctnk))]
adtsce <- SingleCellExperiment(assays=list(counts=cnts, logcounts=adtclrgeoss), 
                               colData=cdctnk)
library(scran)
adtsce <- runPCA(adtsce)
adtsce <- runTSNE(adtsce, dimred="PCA")
plotReducedDim(adtsce, dimred="TSNE", colour_by="celltype")

library(MultiAssayExperiment)
library(rhdf5)
library(HDF5Array)

ddir <- "~/data/scmm/pbmc_10x"

pbmc <- readRDS(
    file.path(ddir, "mae.rds")
)
vdir <- file.path(ddir, paste0("v", "1.0.0"))
setwd(vdir)

## save colData and sampleMap
pbmc_colData <- colData(pbmc)
save(pbmc_colData, file = "pbmc_colData.rda")

pbmc_sampleMap <- sampleMap(pbmc)
save(pbmc_sampleMap, file = "pbmc_sampleMap.rda")

Matrix::writeMM(assay(pbmc[[1]]), "pbmc_rna.mtx")
R.utils::gzip(filename = "pbmc_rna.mtx", destname = "pbmc_rna.mtx.gz")
file.remove("pbmc_rna.mtx")

stopifnot(file.exists("pbmc_rna.mtx.gz"))

rna_mtx <- .read_mtx("pbmc_rna.mtx.gz")

## save H5 file and SCE shell
HDF5Array::saveHDF5SummarizedExperiment(pbmc[[1]], dir = "pbmc_rna",
    prefix = "pbmc_rna_", as.sparse = TRUE)

## load SCE shell
rna_sce <- readRDS("./pbmc_rna/pbmc_rna_se.rds")
## replace assay with MTX assay
pbmc_rna_mtx_obj <- BiocBaseUtils::setSlots(
    rna_sce, assays = Assays(SimpleList(counts = rna_mtx))
)

pbmc_rna_h5_obj <-
    HDF5Array::loadHDF5SummarizedExperiment("pbmc_rna", "pbmc_rna_")

Matrix::writeMM(assay(pbmc[[2]]), "pbmc_atac.mtx")
R.utils::gzip(filename = "pbmc_atac.mtx", destname = "pbmc_atac.mtx.gz")
pbmc_atac_mtx <- "pbmc_atac.mtx.gz"
stopifnot(file.exists(pbmc_atac_mtx))
atac_mtx <- .read_mtx("pbmc_atac.mtx.gz")
## save H5 file and SCE shell
HDF5Array::saveHDF5SummarizedExperiment(pbmc[[2]], dir = "pbmc_atac",
    prefix = "pbmc_atac_", as.sparse = TRUE)

## load SCE shell
atac_sce <- readRDS("./pbmc_atac/pbmc_atac_se.rds")
## replace assay with MTX assay
pbmc_atac_mtx_obj <- BiocBaseUtils::setSlots(
    atac_sce, assays = Assays(SimpleList(counts = atac_mtx))
)

## load H5 object
pbmc_atac_h5_obj <-
    HDF5Array::loadHDF5SummarizedExperiment("pbmc_atac", "pbmc_atac_")

### Use 10X Dataset
## First load the previous version
rna <- SingleCellMultiModal::scMultiome(
    "pbmc_10x", "rna", format = "HDF5", version = "1.0.0", dry.run = FALSE
)

rnames <- rownames(rna[[1]])
cnames <- colnames(rna[[1]])

datafolder <- file.path(Sys.getenv("HOME"), "data/scmm/pbmc_10x")

v1url <- paste0("https://cf.10xgenomics.com/samples/cell-arc/1.0.0/",
"pbmc_granulocyte_sorted_10k/pbmc_granulocyte_sorted_10k_filtered_feature_bc_matrix.h5"
)

h5v1 <- file.path(datafolder, basename(v1url))

if (!file.exists(h5v1))
    download.file(
        url = v1url,
        destfile = h5v1
    )

HDF5Array::h5ls(h5v1, all = TRUE)

grange <- as.character(h5read(h5v1, "/matrix/features/name"))

aa <- HDF5Array::TENxMatrix(h5v1, "matrix")
rownames(aa) <- grange
upcnames <- gsub("(.*)(-1)$", "\\1", colnames(aa))

stopifnot(
    all.equal(
        nchar(colnames(aa)) - nchar("-1") , nchar(upcnames)
    )
)

colnames(aa) <- upcnames

rowlog <- grange %in% rnames
aa <- aa[rowlog,]

collog <- colnames(aa) %in% cnames
aa <- aa[, collog]

all(rownames(aa) %in% rnames)
## TRUE
all(colnames(aa) %in% cnames)
## TRUE

## remove dups HDF5Array
duprows <- duplicated(rownames(aa))
aa <- aa[!duprows, ]

## make sure that previous and new values are the same
## identical(rowSums(assay(rna[[1]])), rowSums(aa))

HDF5Array::writeTENxMatrix(aa, outfile, "pbmc", verbose = TRUE)
# file.remove(outfile)
outfile

HDF5Array::h5ls(outfile, all = TRUE)

SCMM="$HOME/gh/SingleCellMultiModal"

WIKI="$HOME/wiki/SingleCellMultiModal.wiki"

RVER="devel"

cd $SCMM

export R_LIBS_USER="/media/$USER/1D24A0EA4286043C1/bioc-$RVER/"

RCMD="$HOME/src/svn/r-$RVER/R/bin/R --no-save --no-restore-data"

$RCMD CMD INSTALL $SCMM

$RCMD -e "rmarkdown::render('inst/scripts/Contributing-Guidelines.Rmd', output_file = '$WIKI/Contributing-Guidelines.md')"

cd $WIKI

git diff

git pull origin master
git commit -am "update wiki"
git push origin master

---
title: "SCoPE2: macrophage vs monocytes"
author: "Christophe Vanderaa"
date: "`r BiocStyle::doc_date()`"
vignette: |
    %\VignetteIndexEntry{SCoPE2: macrophage vs monocytes}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
output:
    BiocStyle::html_document:
        toc_float: true
package: SingleCellMultiModal
bibliography: ../inst/REFERENCES.bib
---

This vignette will guide you through how accessing and manipulating
the SCoPE2 data sets available from the `SingleCellMultimodal` package.

# Installation

```{r,eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("SingleCellMultiModal")
```

## Load packages

```{r,include=TRUE,results="hide",message=FALSE,warning=FALSE}
library(SingleCellMultiModal)
library(MultiAssayExperiment)
```

# SCoPE2

SCoPE2 is a mass spectrometry (MS)-based single-cell proteomics
protocol to quantify the proteome of single-cells in an untargeted
fashion. It was initially developed by @Specht2021-pm.

## Downloading data sets

The user can see the available data set by using the default options.

```{r}
SCoPE2("macrophage_differentiation",
       mode = "*",
       version = "1.0.0",
       dry.run = TRUE)
```

Or by simply running:

```{r}
SCoPE2("macrophage_differentiation")
```

## Available projects

Currently, only the `macrophage_differentiation` is available.

## Retrieving data

You can retrieve the actual data from `ExperimentHub` by setting
`dry.run = FALSE`. This example retrieves the complete data set
(transcriptome and proteome) for the `macrophage_differentiation`
project:

```{r,message=FALSE}
scope2 <- SCoPE2("macrophage_differentiation",
                 modes = "rna|protein",
                 dry.run = FALSE)
scope2
```

# The macrophage differentiation project

This data set has been acquired by the Slavov Lab (@Specht2021-pm).
It contains single-cell proteomics and single-cell
RNA sequencing data for macrophages and monocytes. The objective of the
research that led to generate the data is to understand whether
homogeneous monocytes differentiate in the absence of cytokines to
macrophages with homogeneous or heterogeneous profiles. The transcriptomic and
proteomic acquisitions are conducted on two separate subset of similar
cells (same experimental design). The cell type of the samples are known only
for the **proteomics** data. The proteomics data was retrieved from
the authors' [website](https://scope2.slavovlab.net/docs/data) and the
transcriptomic data was retrieved from the GEO database (accession id:
[GSE142392](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE142392)).

For more information on the protocol, see @Specht2021-pm.

## Data versions

Only version `1.0.0` is currently available.

The `macrophage_differentiation` data set in this package contains two
assays: `rna` and `protein`.

### Cell annotation

The single-cell proteomics data contains cell type annotation
(`celltype`), sample preparation batch (`batch_digest` and
`batch_sort`), chromatography batch (`batch_chromatography`), and the
MS acquisition run (`batch_MS`). The single-cell transcriptomics data
was acquired in two batches (`batch_Chromium`). Note that because the
cells that compose the two assays are distinct, there is no common
cell annotation available for both proteomics and transcriptomics. The
annotation were therefore filled with `NA`s accordingly.

```{r}
colData(scope2)
```

### Transcriptomic data

You can extract and check the transcriptomic data through subsetting:

```{r}
scope2[["rna"]]
```

The data is rather large and is therefore stored on-disk using the
HDF5 backend. You can verify this by looking at the assay data matrix.
Note that the counts are UMI counts.

```{r}
assay(scope2[["rna"]])[1:5, 1:5]
```

### Proteomic data

The `protein` assay contains MS-based proteomic data.
The data have been passed sample and feature quality control,
normalized, log transformed, imputed and batch corrected. Detailed
information about the data processing is available in
[another vignette](https://uclouvain-cbio.github.io/SCP.replication/articles/SCoPE2.html). You can extract the proteomic data similarly to the
transcriptomic data:

```{r}
scope2[["protein"]]
```

In this case, the protein data have reasonable size and are loaded
directly into memory. The data matrix is stored in `logexprs`. We
decided to not use the traditional `logcounts` because MS proteomics
measures intensities rather than counts as opposed to scRNA-Seq.

```{r}
assay(scope2[["protein"]])[1:5, 1:5]
```

# sessionInfo

```{r}
sessionInfo()
```

# References

---
output: github_document
date: "`r format(Sys.time(), '%B %d, %Y')`"
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  cache = TRUE,
  out.width = "100%"
)
options(tibble.print_min = 5, tibble.print_max = 5)
```

# Overview

Thank you for your interest!

The `SingleCellMultiModal` package aims to provide single cell datasets
from several different technologies / modalities for benchmarking and analysis.
We currently provide from `scNMT`, `scM&T`, `seqFISH`, `CITEseq`, and other
technologies. Contributions are very much welcome.

# List of Multi-modal Datasets

For a full list of available datasets, see here:
[Google Drive Sheet](https://docs.google.com/spreadsheets/d/14Eq_Bt_3tKx_t1UDwan0qZZGWNyK-d2WLXtmoPGB5To/edit#gid=0)

# Contributing

In order to contribute, we generally require data in `Rda` or `Rds` format
though we also support `HDF5` and `MTX` formats. Aside from the usual required
`metadata.csv` documentation in the package, contributors are required to add a
name to the `DataType` column in the metadata table that indicates the name of
the contributed dataset. To illustrate, here are some `DataType` names already
in the package:

* mouse_gastrulation
* mouse_visual_cortex
* cord_blood
* pbmc_10x
* macrophage_differentiation
* mouse_embryo_8_cell

```{r,include=TRUE,results="hide",message=FALSE,warning=FALSE}
library(SingleCellMultiModal)
```

```{r}
meta <- system.file("extdata", "metadata.csv",
    package = "SingleCellMultiModal", mustWork = TRUE)
head(read.csv(meta))
```

# Versioning and folder structure

We associate a version with all datasets. We start with version `1.0.0` using
semantic versioning and include data in a corresponding version folder
(`v1.0.0`). Thus, the recommended folder structure is as follows:

```
~/data
  └ scmm/
    └ mouse_gastrulation/
      └ v1.0.0/
        └ scnmt_acc_cgi.rda
        └ scnmt_met_genebody.rda
        └ scnmt_met_cgi.rda
        └ scnmt_rna.rda
        └ scnmt_colData.rda
        └ scnmt_sampleMap.rda
```

In the `inst` section, we will discuss how to annotate these data products.

# Files

It is customary to include one `Rda` / `Rds` file per assay or per assay and
region combination of interest (as above). We also highly recommend including
`sampleMap` and `colData` datasets for the `MultiAssayExperiment` that will
be built on the fly. In this example, there are three modalities in the `scNMT`
dataset, `rna` (transcriptome), `acc` (chromatin accessibility), and `met`
(methylation).

# vignettes

Contributors are required to demonstrate user-level functionality via
examples in a vignette for each contributed dataset.

# R

Ideally, the interface for the contributed dataset should be similar to that
of `scNMT` so that users have a sense of consistency in the usage of the
package. This means having one main function that returns a
`MultiAssayExperiment` object and having options that show the user what
datasets are available for a particular technology. Contributors should use
`roxygen2` for documenting datasets and using `@inheritParams scNMT` tag
to avoid copying `@param` documentation.

See the current example for implementation details:

```{r}
scNMT(
    DataType = "mouse_gastrulation",
    mode = "*",
    version = "1.0.0",
    dry.run = TRUE
)
```

**Note**. Contributors should ensure that the documentation is complete and the
proper data sources have been attributed.

# inst/*

## extdata/

In the following section we will describe how to annotate and append to
the `metadata.csv` file. First, we have to ensure that we are accounting for
all of the fields required by `ExperimentHub`. They are listed here:

* ResourceName
* Title
* Description
* BiocVersion
* Genome
* SourceType
* SourceUrl
* SourceVersion
* Species
* TaxonomyId
* Coordinate_1_based
* DataProvider
* Maintainer
* RDataPath
* RDataClass
* DispatchClass
* DataType+

**Note**. `DataType` is a field we've added to help distinguish multimodal
technologies and is required for `SingleCellMultiModal`. Some of the
`DataType`s already available are `mouse_gastrulation`, `mouse_visual_cortex`,
`cord_blood`, `peripheral_blood`, etc.

To make it easy for contributions, we've provided a mechanism for easy
documentation using a file from a `data.frame` we call a `doc_file`.

Interested contributors should create a `doc_file` in `inst/extdata/docuData`
folder. Although we do not have a strict naming convention for the `doc_file`,
we usually name the file `singlecellmultimodalvX.csv` where `X` is the *n*th
dataset added to the package.

Here is an example of the file from version `v1.0.0` of the `scNMT` dataset:

```{r}
doc_file <- system.file("extdata", "docuData", "singlecellmultimodalv1.csv",
    package = "SingleCellMultiModal", mustWork = TRUE)
read.csv(doc_file, header = TRUE)
```

Contributors will then use their `doc_file` to append to the existing
`metadata.csv`.

To create a `doc_file` `data.frame` with the file name
`singlecellmultimodalvX.csv`, first we create a `data.frame` object.
Each general annotation or row in this `data.frame` will be applied to all
files uploaded to `ExperimentHub`. We take advantage of the `data.frame`
function to repeat data and create a uniform `data.frame` with equal values
across the columns.

```{r}
scmeta <- data.frame(
    DataProvider =
        "Dept. of Bioinformatics, The Babraham Institute, United Kingdom",
    TaxonomyId = "10090",
    Species = "Mus musculus",
    SourceUrl = "https://cloudstor.aarnet.edu.au/plus/s/Xzf5vCgAEUVgbfQ",
    SourceType = "RDS",
    SourceVersion = "1.0.0",
    DataType = "mouse_gastrulation",
    Maintainer = "Ricard Argelaguet <ricard@ebi.ac.uk>",
    stringsAsFactors = FALSE
)
scmeta
```

### Saving the data

After creating the documentation `data.frame` (`doc_file`), the contributor can
save that dataset as a `.csv` file using `write.csv`.

```{r,eval=FALSE}
write.csv(
    scmeta,
    file = "inst/extdata/docuData/singlecellmultimodal.csv",
    row.names = FALSE
)
```

## Documenting diverse data

In the case that the contributed data is not uniform, meaning that there are
multiple file types from potentially different speciments, the `data.frame`
will have to account for _all_ contributed data files.

For example, if the contributed data has a number of different source types,
the contributor is required to create a `data.frame` with the number of rows
equal to the number of files to be uploaded.

In this example, we have **two** data files from different source types and
formats:

```{r}
data.frame(
    DataProvider =
        c("Institute of Population Genetics", "Mouse Science Center"),
    TaxonomyId = c("9606", "10090"),
    Species = c("Homo sapiens", "Mus musculus"),
    SourceUrl = c("https://human.science/org", "https://mouse.science/gov"),
    SourceType = c("RDS", "XML"),
    DataType = c("human_genetics", "mouse_genetics"),
    stringsAsFactors = FALSE
)
```

## scripts/

### make-data/

The individual data products that will eventually come together into
a `MultiAssayExperiment` can be uploaded as serialized `RDA` / `RDS` files,
`HDF5`, and even `MTX` files. For examples on how to save data into
their respective file formats, see the `make-data` folder.

## Generating the metadata.csv

### make-metadata.R

Based on the folder structure described previously, the `directory` argument in
`make_metadata` will correspond to the `~/data/scmm` folder. The `dataDir`
folder will correspond to the `DataType` / technology subfolder (e.g.,
"mouse_gastrulation"). These will be used as inputs to the `make_metadata`
function.

Once the data is ready, the user can use the function in `make-metadata.R`
in the `scripts` folder.  A typical call to `make_metadata` will either add to
the metadata or replace it entirely. The easiest for current contributors is to
`append` rows to the metadata file.

```{r,eval=FALSE}
make_metadata(
    directory = "~/data/scmm",
    dataDirs = "mouse_gastrulation", # also the name of the DataType
    ext_pattern = "\\.[Rr][Dd][Aa]$",
    doc_file = "inst/extdata/docuData/singlecellmultimodalv1.csv",
    pkg_name = "SingleCellMultiModal",
    append = TRUE,
    dry.run = TRUE
)
```

Note that the extraction pattern (`ext_pattern`) will allow contributors to
match a specific file extension in that folder and ignore any intermediate
files.

The contributor may also wish to run `dry.run=TRUE` to see the output
`data.frame` to be added to the `metadata.csv` file.

_Note_. The `make_metadata` function should be run from the base package
directory from a GitHub / git checkout (`git clone ...`).

## Validation

It is recommended to run the metadata validation function from
`AnnotationHubData`:

```{r,eval=FALSE}
AnnotationHubData::makeAnnotationHubMetadata("SingleCellMultiModal")
```

to ensure that some of the metadata fields are properly annotated.


# NEWS.md

Contributors should update the `NEWS.md` file with a mention of the
function and data that are being provided. See the `NEWS.md` for examples.

# Next steps

The contributor should then create a Pull Request on [GitHub][].

[GitHub]: https://github.com/waldronlab/SingleCellMultiModal/pulls

If you are interested in contributing, I can help you go over the contribution
and submission. Please contact me either on the [Bioc-community Slack][]
(mramos148) or at marcel {dot} ramos [at] sph (dot) cuny (dot) edu.
If you need to sign up to the community Slack channel, follow this link:
https://bioc-community.herokuapp.com/

[Bioc-community Slack]: https://community-bioc.slack.com

## sessionInfo

<details>
  <summary> sessionInfo </summary>

```{r,echo=FALSE}
sessionInfo()
```

</details>

---
title: "seqFISH Mouse Visual Cortex"
author: "Dario Righelli"
date: "`r format(Sys.time(), '%d %B, %Y')`"
output:
    BiocStyle::html_document:
      toc_float: true
vignette: >
    %\VignetteIndexEntry{seqFISH Mouse Visual Cortex}
    %\VignetteEncoding{UTF-8}
    %\VignetteEngine{knitr::rmarkdown}
package: SingleCellMultiModal
bibliography: ../inst/REFERENCES.bib
editor_options:
  chunk_output_type: console
---

# Installation

```{r,eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("SingleCellMultiModal")
```

## Load packages

```{r,include=TRUE, results="hide", message=FALSE, warning=FALSE}
library(MultiAssayExperiment)
library(SpatialExperiment)
library(SingleCellMultiModal)
```


# seq-FISH dataset

The dataset consists of two data types,
seq-FISH data was provided by @Zhu2018identification, while scRNA-seq data
was provided by @Tasic2016adult.

Data have been retrievedas part of the
[Hackathon](https://github.com/BIRSBiointegration/Hackathon/tree/master/seqFISH)
in the
[Mathematical Frameworks for Integrative Analysis of Emerging Biological DataTypes](https://www.birs.ca/events/2020/5-day-workshops/20w5197) workshop.

## Downloading datasets

The user can see the available dataset by using the default options

```{r}
seqFISH(
    DataType="mouse_visual_cortex", modes="*", dry.run=TRUE, version="2.0.0"
)
```

Or simply by running:

```{r}
seqfish <- seqFISH(
    DataType="mouse_visual_cortex", modes="*", dry.run=FALSE, version="2.0.0"
)
seqfish
```

Extract the list of experiments _without_ the associated colData.

```{r}
experiments(seqfish)
```

## Exploring the data structure

Check row annotations for all experiments:

```{r}
rownames(seqfish)
```

Take a peek at the `sampleMap` (graph representation of assays, cells, and
barcodes):

```{r}
sampleMap(seqfish)
```

## Visualize matching cell identifiers across assays

```{r}
upsetSamples(seqfish)
```

This shows that about 1597 cells match across both modalities / assays.

## scRNA-seq data

The scRNA-seq data are accessible with `$scRNAseq`, which returns a
*SingleCellExperiment* class object, with all its associated methods.

```{r}
seqfish[["scRNAseq"]]
```

Otherwhise the `assay` function can be used to access the *scRNAseq* assay
stored in the `seqfish` *MultiAssayExperiment* object.

```{r}
head(assay(seqfish, "scRNAseq"))[,1:4]
```

## seq-FISH data

The seq-FISH data are accessible with `$seqFISH`, which returns a
**SpatialExperiment** class object.

```{r}
seqfish[["seqFISH"]]
```

Otherwhise the `assay` function can be used to access the *seqFISH* assay
stored in the `seqfish` *MultiAssayExperiment* object.

```{r}
head(assay(seqfish, "seqFISH"))[,1:4]
```

Spatial data can be retrieved with `spatialData` function on the
*SpatialExperiment* object.

```{r}
(sd <- spatialData(seqfish[["seqFISH"]]))
```

Spatial coordinates within the spatial data can be retrieved in matrix form
with `spatialCoords` function on the *SpatialExperiment* object.

```{r}
head(sc <- spatialCoords(seqfish[["seqFISH"]]))
```

Direct access to the colnames of the spacial coordinates with
`spatialCoordsNames` function.

```{r}
spatialCoordsNames(seqfish[["seqFISH"]])
```


## Other data version

The provided seqFISH dataset comes out in two different versions:

* 1.0.0 - provides the same seqFISH data as shown in the rest of this
vignette, but it returns the full normalized scRNA-seq data matrix (with
labels), as released from the original authors on the GEO database.
* 2.0.0 - provides the same seqFISH data as shown in the rest of this
vignette, but it returns a processed subset of the original scRNA-seq data,
providing only the same genes present in the seqFISH data matrix.

### Data version 1.0.0

The full scRNA-seq data matrix is 24057 rows x 1809 columns.

To access the v1.0.0 simply run

```{r}
seqFISH(
    DataType="mouse_visual_cortex", modes="*", dry.run=FALSE, version="1.0.0"
)
```

# Session Info

```{r, tidy=TRUE}
sessionInfo()
```

---
title: "SingleCellMultiModal Introduction"
date: "`r BiocStyle::doc_date()`"
vignette: |
  %\VignetteIndexEntry{SingleCellMultiModal Introduction}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output:
    BiocStyle::html_document:
      toc_float: true
package: SingleCellMultiModal
bibliography: ../inst/REFERENCES.bib
---

# SingleCellMultiModal

## Overview 

`SingleCellMultiModal` is an R package that provides a convenient and
user-friendly representation of multi-modal data using `MultiAssayExperiment`.
This package introduces a suite of single-cell multimodal landmark datasets for
benchmarking and testing multimodal analysis methods via the `ExperimentHub`
Bioconductor package. The scope of this package is to provide efficient access
to a selection of curated, pre-integrated, publicly available landmark datasets
for methods development and benchmarking.

## Installation

```{r,eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("SingleCellMultiModal")
```

## Loading packages

```{r,include=TRUE,results="hide",message=FALSE,warning=FALSE}
library(SingleCellMultiModal)
library(MultiAssayExperiment)
```

# Citing SingleCellMultiModal

Your citations are crucial in keeping our software free and open source. To
cite our package see the citation (@Eckenrode2023-yq) in the Reference
section. You may also browse to the publication at
[PLoS Computational Biology][1].

[1]: https://doi.org/10.1371/journal.pcbi.1011324

## Representation

Users can obtain integrative representations of multiple modalities as a
`MultiAssayExperiment`, a common core Bioconductor data structure relied on by
dozens of multimodal data analysis packages. `MultiAssayExperiment` harmonizes
data management of multiple experimental assays performed on an overlapping set
of specimens. Although originally developed for patient data from multi-omics
cancer studies, the `MultiAssayExperiment` framework naturally applies also to
single cells. A schematic of the data structure can be seen below. In this
context, "patients" are replaced by "cells". We use `MultiAssayExperiment`
because it provides a familiar user experience by extending
`SummarizedExperiment` concepts and providing open ended compatibility with
standard data classes present in Bioconductor such as the
`SingleCellExperiment`.

```{r,echo=FALSE}
imgurl <- paste0(
    "https://github.com/waldronlab/MultiAssayExperiment/blob/",
    "c3c59a094e5a08111ee98b9f69579db5634d9fd4/vignettes/",
    "MultiAssayExperiment.png?raw=true"
)
knitr::include_graphics(
    path = imgurl
)
```

# Contributions

Want to contribute to the `SingleCellMultiModal` package? We welcome
contributions from the community. Please refer to our
[Contributing Guidelines][2] for more details.

[2]: https://github.com/waldronlab/SingleCellMultiModal/wiki/Contributing-Guidelines


## Further resources

For more information on the `MultiAssayExperiment` data structure, please refer
to @Ramos2017-tk as well as the [MultiAssayExperiment vignette][3].

[3]: https://bioconductor.org/packages/release/bioc/vignettes/MultiAssayExperiment/inst/doc/MultiAssayExperiment.html

# References

---
title: "scNMT Mouse Gastrulation"
author: 
  - name: "Marcel Ramos Pérez"
date: "`r BiocStyle::doc_date()`"
vignette: |
  %\VignetteIndexEntry{scNMT Mouse Gastrulation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output:
    BiocStyle::html_document:
      toc_float: true
package: SingleCellMultiModal
bibliography: ../inst/REFERENCES.bib
---

# Installation

```{r,eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("SingleCellMultiModal")
```

## Load packages

```{r,include=TRUE,results="hide",message=FALSE,warning=FALSE}
library(SingleCellMultiModal)
library(MultiAssayExperiment)
```

# scNMT: single-cell nucleosome, methylation and transcription sequencing

The dataset was graciously provided by @Argelaguet2019-et.

Scripts used to process the raw data were written and maintained by Argelaguet
and colleagues and reside on GitHub:
https://github.com/rargelaguet/scnmt_gastrulation

For more information on the protocol, see @Clark2018-qg.

## Dataset lookup

The user can see the available datasets by using the `dry.run` argument:

```{r}
scNMT("mouse_gastrulation", mode = "*", version = "1.0.0", dry.run = TRUE)
```

Or by simply running the `scNMT` function with defaults:

```{r}
scNMT("mouse_gastrulation", version = "1.0.0")
```

## Data versions

A more recent release of the 'mouse_gastrulation' dataset was provided
by Argelaguet and colleagues. This dataset includes additional cells that
did not pass the original quality metrics as imposed for the version `1.0.0`
dataset.

Use the `version` argument to indicate the newer dataset version
(i.e., `2.0.0`):

```{r}
scNMT("mouse_gastrulation", version = '2.0.0', dry.run = TRUE)
```

## Downloading the data

To obtain the data, we can use the `mode` argument to indicate specific
datasets using 'glob' patterns that will match the outputs above. For example,
if we would like to have all 'genebody' datasets for all available assays,
we would use `*_genebody` as an input to `mode`.

```{r,message=FALSE}
nmt <- scNMT("mouse_gastrulation", mode = c("*_DHS", "*_cgi", "*_genebody"),
    version = "1.0.0", dry.run = FALSE)
nmt
```

## Checking the cell metadata

Included in the `colData` `DataFrame` within the `MultiAssayExperiment` class
are the variables `cellID`, `stage`, `lineage10x_2`, and `stage_lineage`.
To extract this `DataFrame`, one has to use `colData` on the
`MultiAssayExperiment` object:

```{r}
colData(nmt)
```

## Exploring the data structure

Check row annotations:

```{r}
rownames(nmt)
```

The `sampleMap` is a graph representation of the relationships between cells
and 'assay' datasets:

```{r}
sampleMap(nmt)
```

Take a look at the cell identifiers or barcodes across assays:

```{r}
colnames(nmt)
```

## Chromatin Accessibility (acc_*)

See the accessibilty levels (as proportions) for DNase Hypersensitive Sites:

```{r}
head(assay(nmt, "acc_DHS"))[, 1:4]
```

## DNA Methylation (met_*)

See the methylation percentage / proportion:

```{r}
head(assay(nmt, "met_DHS"))[, 1:4]
```

For protocol information, see the references below.

# sessionInfo

```{r}
sessionInfo()
```

# References

---
title: "CITEseq Cord Blood"
author: "Dario Righelli"
date: "`r format(Sys.time(), '%d %B, %Y')`"
output:
    BiocStyle::html_document:
      toc_float: true
vignette: >
    %\VignetteIndexEntry{CITEseq Cord Blood}
    %\VignetteEncoding{UTF-8}
    %\VignetteEngine{knitr::rmarkdown}
package: SingleCellMultiModal
bibliography: ../inst/REFERENCES.bib
editor_options:
  chunk_output_type: console
---

# Installation 

```{r,eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("SingleCellMultiModal")
```


# Load libraries

```{r, include=TRUE, results="hide", message=FALSE, warning=FALSE}

library(MultiAssayExperiment)
library(SingleCellMultiModal)
library(SingleCellExperiment)
```


# CITE-seq dataset

CITE-seq data are a combination of two data types extracted at the same
time from the same cell.  First data type is scRNA-seq data, while the second
one consists of about a hundread of antibody-derived tags (ADT).
In particular this dataset is provided by @stoeckius2017simultaneous.

## Downloading datasets

The user can see the available dataset by using the default options

```{r}

CITEseq(DataType="cord_blood", modes="*", dry.run=TRUE, version="1.0.0")

```

Or simply by setting `dry.run = FALSE` it downloads the data and creates the
`MultiAssayExperiment` object.

In this example, we will use one of the two available datasets `scADT_Counts`:

```{r,message=FALSE}

mae <- CITEseq(
    DataType="cord_blood", modes="*", dry.run=FALSE, version="1.0.0"
)

mae
```

Example with actual data:

```{r}
experiments(mae)
```


## Exploring the data structure

Check row annotations:

```{r}
rownames(mae)
```

Take a peek at the `sampleMap`:

```{r}
sampleMap(mae)
```


## scRNA-seq data

The scRNA-seq data are accessible with the name `scRNAseq`, which returns a
*matrix* object.

```{r}
head(experiments(mae)$scRNAseq)[, 1:4]
```

## scADT data

The scADT data are accessible with the name `scADT`, which returns a
**matrix** object.

```{r}
head(experiments(mae)$scADT)[, 1:4]
```

# SingleCellExperiment object conversion

Because of already large use of some methodologies (such as
in the [SingleCellExperiment vignette][1] or [CiteFuse Vignette][2] where the
`SingleCellExperiment` object is used for CITE-seq data,
we provide a function for the conversion of our CITE-seq `MultiAssayExperiment`
object into a `SingleCellExperiment` object with scRNA-seq data as counts and
scADT data as `altExp`s.

[1]: https://www.bioconductor.org/packages/release/bioc/vignettes/SingleCellExperiment/inst/doc/intro.html#5_adding_alternative_feature_sets
[2]: http://www.bioconductor.org/packages/release/bioc/vignettes/CiteFuse/inst/doc/CiteFuse.html

```{r message=FALSE}
sce <- CITEseq(DataType="cord_blood", modes="*", dry.run=FALSE, version="1.0.0",
              DataClass="SingleCellExperiment")
sce
```

# Session Info

```{r, tidy=TRUE}
sessionInfo()
```

# References

---
title: "G&T-seq Mouse Embryo (8-cell stage)"
author: "Ludwig Geistlinger"
date: "`r BiocStyle::doc_date()`"
vignette: |
  %\VignetteIndexEntry{GT-seq Mouse Embryo}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output:
    BiocStyle::html_document:
      toc_float: true
package: SingleCellMultiModal
bibliography: ../inst/REFERENCES.bib
---

# Installation

```{r,eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("SingleCellMultiModal")
```

## Load

```{r,include=TRUE,results="hide",message=FALSE,warning=FALSE}
library(SingleCellMultiModal)
library(MultiAssayExperiment)
```

# G&T-seq: parallel sequencing data of single-cell genomes and transcriptomes

G&T-seq is a combination of Picoplex amplified gDNA sequencing (genome) and
SMARTSeq2 amplified cDNA sequencing (transcriptome) of the same cell.
For more information, see @Macaulay2015.

## Downloading datasets

The user can see the available dataset by using the default options

```{r}
GTseq("mouse_embryo_8_cell", mode = "*", dry.run = TRUE)
```

Or by simply running:

```{r}
GTseq()
```

## Obtaining the data

To obtain the actual datasets:

```{r,message=FALSE}
gts <- GTseq(dry.run = FALSE)
gts
```

## Exploring the data structure

Check available metadata for each of the 112 mouse embryo cells assayed by G&T-seq:

```{r}
colData(gts)
```

Take a peek at the `sampleMap`:

```{r}
sampleMap(gts)
```

## Copy numbers

To access the integer copy numbers as detected from scDNA-seq:

```{r}
head(assay(gts, "genomic"))[, 1:4]
```

## RNA-seq

To access raw read counts as quantified from scRNA-seq:

```{r}
head(assay(gts, "transcriptomic"))[, 1:4]
```

For protocol information, see @Macaulay2016.

# sessionInfo

```{r}
sessionInfo()
```

# References

---
title: "ECCITEseq Peripheral Blood"
author: "Dario Righelli"
date: "`r format(Sys.time(), '%d %B, %Y')`"
output:
    BiocStyle::html_document:
      toc_float: true
vignette: >
    %\VignetteIndexEntry{ECCITEseq Peripheral Blood}
    %\VignetteEncoding{UTF-8}
    %\VignetteEngine{knitr::rmarkdown}
package: SingleCellMultiModal
bibliography: ../inst/REFERENCES.bib
editor_options:
  chunk_output_type: console
---

# Installation 

```{r,eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("SingleCellMultiModal")
```


# Load libraries

```{r, include=TRUE, results="hide", message=FALSE, warning=FALSE}

library(MultiAssayExperiment)
library(SingleCellMultiModal)
library(SingleCellExperiment)

```


# ECCITE-seq dataset

ECCITE-seq data are an evolution of the CITE-seq data 
(see also [CITE-seq vignette](CITEseq.html) for more details)
by extending the CITE-seq original data types with a third one always extracted
from the same cell.
Indeed, in addition to the CITE-seq providing scRNA-seq and antibody-derived tags
(ADT), it provides around ten Hashtagged Oligo (HTO).
In particular this dataset is provided by @mimitou2019multiplexed.

## Downloading datasets

The user can see the available dataset by using the default options through the
CITE-seq function.

```{r}

CITEseq(DataType="peripheral_blood", modes="*", dry.run=TRUE, version="1.0.0")

```

Or simply by setting `dry.run = FALSE` it downloads the data and by default 
creates the `MultiAssayExperiment` object.

In this example, we will use one of the two available datasets `scADT_Counts`:

```{r message=FALSE}

mae <- CITEseq(DataType="peripheral_blood", modes="*", dry.run=FALSE, version="1.0.0")
mae
```

Example with actual data:

```{r}
experiments(mae)
```

Additionally, we stored into the object metedata 

## Exploring the data structure

Check row annotations:

```{r}
rownames(mae)
```

Take a peek at the `sampleMap`:

```{r}
sampleMap(mae)
```


## scRNA-seq data

The scRNA-seq data are accessible with the name `scRNAseq`, which returns a
*matrix* object.

```{r}
head(experiments(mae)$scRNA)[, 1:4]
```

## scADT data

The scADT data are accessible with the name `scADT`, which returns a
**matrix** object.

```{r}
head(experiments(mae)$scADT)[, 1:4]
```

## CTCL/CTRL conditions

The dataset has two different conditions (CTCL and CTRL) which samples can be identified with the `colData` accessor.

CTCL stands for cutaneous T-cell lymphoma while CTRL for control.

For example, if we want only the CTCL samples, we can run:

```{r}
(ctclMae <- mae[,colData(mae)$condition == "CTCL",])
```

And if you're interested into the common samples across all the modalities
you can use the `complete.cases` funtion.

```{r}
ctclMae[,complete.cases(ctclMae),]
```

 
## sgRNAs CRISPR pertubation data

The CRISPR perturbed scRNAs data are stored in a different spot 
to keep their original long format.

They can be accessed with the `metadata` accessors which, in this case returns a named `list` of `data.frame`s.

```{r}
sgRNAs <- metadata(mae)
names(sgRNAs)
```

There are four different sgRNAs datasets, one per each condition and family receptors combination.

TCR stands for T-Cell Receptor, while a,b,g,d stand for alpha, beta, gamma and delta respectively.

To look into the TCRab, simply run:

```{r}
head(sgRNAs$CTCL_TCRab)
```

# SingleCellExperiment object conversion

Because of already large use of some methodologies (such as
in the [SingleCellExperiment vignette][1] or [CiteFuse Vignette][2] where the
`SingleCellExperiment` object is used for CITE-seq data,
we provide a function for the conversion of our CITE-seq `MultiAssayExperiment`
object into a `SingleCellExperiment` object with scRNA-seq data as counts and
scADT data as `altExp`s.


```{r message=FALSE}
sce <- CITEseq(DataType="peripheral_blood", modes="*", dry.run=FALSE, 
               version="1.0.0", DataClass="SingleCellExperiment")
sce
```

# Session Info

```{r, tidy=TRUE}
sessionInfo()
```

# Additional References

https://www.bioconductor.org/packages/release/bioc/vignettes/SingleCellExperiment/inst/doc/intro.html#5_adding_alternative_feature_sets
http://www.bioconductor.org/packages/release/bioc/vignettes/CiteFuse/inst/doc/CiteFuse.html

# References

---
title: "PBMCs profiled with the Chromium Single Cell Multiome ATAC + Gene Expression from 10x"
author:
  - name: "Marcel Ramos Pérez"
  - name: "Al J Abadi"
  - name: "Ricard Argelaguet"
  - name: "Levi Waldron"
date: "`r BiocStyle::doc_date()`"
vignette: |
  %\VignetteIndexEntry{scMultiome 10x PBMC}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output:
    BiocStyle::html_document:
      toc_float: true
package: SingleCellMultiModal
---

# Installation

```{r,eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("SingleCellMultiModal")
```

## Load

```{r,include=TRUE, results="hide", message=FALSE, warning=FALSE}
library(SingleCellMultiModal)
library(MultiAssayExperiment)
library(scran)
library(scater)
```

# Description

This data set consists of about 10K Peripheral Blood Mononuclear Cells (PBMCs)
derived from a single healthy donor. It is available
[from the 10x Genomics website](https://support.10xgenomics.com/single-cell-multiome-atac-gex/datasets).

Provided are the RNA expression counts quantified at the gene level and the
chromatin accessibility levels quantified at the peak level. Here we provide
the default peaks called by the CellRanger software. If you want to explore
other peak definitions or chromatin accessibility quantifications (at the
promoter level, etc.), you have download the `fragments.tsv.gz` file from the
10x Genomics website.

# Downloading datasets

The user can see the available dataset by using the default options

```{r}
mae <- scMultiome("pbmc_10x", modes = "*", dry.run = FALSE, format = "MTX")
```

```{r, echo=FALSE}
gg_color_hue <- function(n) {
  hues = seq(15, 375, length = n + 1)
  hcl(h = hues, l = 65, c = 100)[1:n]
}
colors <- gg_color_hue(length(unique(mae$celltype)))
names(colors) <- unique(mae$celltype)
```

# Exploring the data structure

There are two assays: `rna` and `atac`, stored as
[SingleCellExperiment](http://bioconductor.org/packages/release/bioc/html/SingleCellExperiment.html)
objects

```{r}
mae
```

where the cells are the same in both assays:

```{r}
upsetSamples(mae)
```

## Cell metadata

Columns:

- **nCount_RNA**: number of read counts
- **nFeature_RNA**: number of genes with at least one read count
- **nCount_ATAC**: number of ATAC read counts
- **nFeature_ATAC**: number of ATAC peaks with at least one read count
- **celltype**: The cell types have been annotated by the 10x Genomics R&D team using gene markers. They provide a rough characterisation of the cell type diversity, but keep in mind that they are not ground truth labels.
- **broad_celltype**: `Lymphoid` or `Myeloid` origin

The cells have not been QC-ed, choosing a minimum number of genes/peaks per
cell depends is left to you! In addition, there are further quality control
criteria that you may want to apply, including mitochondrial coverage, fraction
of reads overlapping ENCODE Blacklisted regions, Transcription start site
enrichment, etc. See suggestions below for software that can perform a
semi-automated quality control pipeline

```{r}
head(colData(mae))
```

## RNA expression

The RNA expression consists of 36,549 genes and 10,032 cells, stored using
the `dgCMatrix` sparse matrix format

```{r}
dim(experiments(mae)[["rna"]])
```

```{r}
names(experiments(mae))
```

Let's do some standard dimensionality reduction plot:

```{r}
sce.rna <- experiments(mae)[["rna"]]

# Normalisation
sce.rna <- logNormCounts(sce.rna)

# Feature selection
decomp <- modelGeneVar(sce.rna)
hvgs <- rownames(decomp)[decomp$mean>0.01 & decomp$p.value <= 0.05]
sce.rna <- sce.rna[hvgs,]

# PCA
sce.rna <- runPCA(sce.rna, ncomponents = 25)

# UMAP
set.seed(42)
sce.rna <- runUMAP(sce.rna, dimred="PCA", n_neighbors = 25, min_dist = 0.3)
plotUMAP(sce.rna, colour_by="celltype", point_size=0.5, point_alpha=1)
```

## Chromatin Accessibility

The ATAC expression consists of 108,344 peaks and 10,032 cells:

```{r}
dim(experiments(mae)[["atac"]])
```

Let's do some standard dimensionality reduction plot. Note that scATAC-seq data is sparser than scRNA-seq, almost binary. The log normalisation + PCA approach that `scater` implements for scRNA-seq is not a good strategy for scATAC-seq data. Topic modelling or TFIDF+SVD are a better strategy. Please see the package recommendations below.

```{r}
sce.atac <- experiments(mae)[["atac"]]

# Normalisation
sce.atac <- logNormCounts(sce.atac)

# Feature selection
decomp <- modelGeneVar(sce.atac)
hvgs <- rownames(decomp)[decomp$mean>0.25]
sce.atac <- sce.atac[hvgs,]

# PCA
sce.atac <- runPCA(sce.atac, ncomponents = 25)

# UMAP
set.seed(42)
sce.atac <- runUMAP(sce.atac, dimred="PCA", n_neighbors = 25, min_dist = 0.3)
plotUMAP(sce.atac, colour_by="celltype", point_size=0.5, point_alpha=1)
```

# Suggested software for the downstream analysis

These are my personal recommendations of R-based analysis software:

-   **RNA expression**: [scater](http://bioconductor.org/packages/release/bioc/html/scater.html), [scran](https://bioconductor.org/packages/release/bioc/html/scran.html)
-   **ATAC accessibility**: [archR](https://www.archrproject.com/), [snapATAC](https://github.com/r3fang/SnapATAC), [cisTopic](https://github.com/aertslab/cisTopic), [Signac](https://satijalab.org/signac), [chromVar](https://bioconductor.org/packages/release/bioc/html/chromVAR.html), [Cicero](https://www.bioconductor.org/packages/release/bioc/html/cicero.html)
-   **Integrative analysis**: [MOFA+](https://biofam.github.io/MOFA2), [Seurat](https://satijalab.org/seurat). Note that both methods have released vignettes in their website where they analysed this same data set.

# sessionInfo

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{scmmCache}
\alias{scmmCache}
\alias{setCache}
\alias{removeCache}
\title{Manage cache / download directories for study data}
\usage{
scmmCache(...)

setCache(
  directory = tools::R_user_dir("SingleCellMultiModal", "cache"),
  verbose = TRUE,
  ask = interactive()
)

removeCache(accession)
}
\arguments{
\item{...}{For \code{scmmCache}, arguments passed to \code{setCache}}

\item{directory}{\code{character(1)} The file location where the cache is located.
Once set, future downloads will go to this folder. See \code{setCache} section
for details.}

\item{verbose}{Whether to print descriptive messages}

\item{ask}{\code{logical(1)} (default TRUE when \code{interactive()}) Confirm the file
location of the cache directory}

\item{accession}{\code{character(1)} A single string indicating the accession number
of the study}
}
\value{
The directory / option of the cache location
}
\description{
Managing data downloads is important to save disk space and
re-downloading data files. This can be done effortlessly via the integrated
\code{BiocFileCache} system.
}
\section{scmmCache}{

Get the directory location of the cache. It will prompt the user to create
a cache if not already created. A specific directory can be used via
\code{setCache}.
}

\section{setCache}{

Specify the directory location of the data cache. By default, it will
go into the user's home and package name directory as given by
\link[tools:R_user_dir]{R_user_dir} (default: varies by system e.g., for Linux:
'$HOME/.cache/R/SingleCellMultiModal').
}

\section{removeCache}{

Some files may become corrupt when downloading, this function allows
the user to delete the tarball associated with a study number in the
cache.
}

\examples{
getOption("scmmCache")
scmmCache()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seqFISH.R
\name{seqFISH}
\alias{seqFISH}
\title{Single-cell spatial + Gene Expression}
\usage{
seqFISH(
  DataType = "mouse_visual_cortex",
  modes = "*",
  version,
  dry.run = TRUE,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{DataType}{\code{character(1)} indicating the identifier of the dataset to
retrieve.  (default "mouse_visual_cortex")}

\item{modes}{\code{character()} The assay types or modes of data to obtain these
include seq-FISH and scRNA-seq data by default.}

\item{version}{\code{character(1)} Either version '1.0.0' or '2.0.0' depending on
data version required (default '1.0.0'). See version section.}

\item{dry.run}{\code{logical(1)} Whether to return the dataset names before actual
download (default \code{TRUE})}

\item{verbose}{\code{logical(1)} Whether to show the dataset currently being
(down)loaded (default \code{TRUE})}

\item{...}{Additional arguments passed on to the
\link[ExperimentHub]{ExperimentHub-class} constructor}
}
\value{
A
\code{\link[MultiAssayExperiment:MultiAssayExperiment-class]{MultiAssayExperiment}}
of seq-FISH data
}
\description{
seqFISH function assembles data on-the-fly from \code{ExperimentHub}
to provide a
\code{\link[MultiAssayExperiment:MultiAssayExperiment-class]{MultiAssayExperiment}}
container. Actually the \code{DataType} argument provides access to the
available datasets associated to the package.
}
\details{
seq FISH data are a combination of single cell spatial coordinates
and transcriptomics for a few hundreds of genes. seq-FISH data can be
combined for example with scRNA-seq data to unveil multiple aspects of
cellular behaviour based on their spatial organization and transcription.

Available datasets are:
\itemize{
\item mouse_visual_cortex: combination of seq-FISH data as obtained from Zhu
et al. (2018) and scRNA-seq data as obtained from Tasic et al. (2016),
Version 1.0.0 returns the full scRNA-seq data matrix, while version 2.0.0
returns the processed and subsetted scRNA-seq data matrix (produced for
the Mathematical Frameworks for Integrative Analysis of Emerging
Biological Data Types 2020 Workshop) The returned seqFISH data are always
the processed ones for the same workshop. Additionally, cell types
annotations are available in the \code{colData} through the \code{class} column in
the seqFISH \code{assay}.
\itemize{
\item scRNA_Counts - Tasic scRNA-seq gene count matrix
\item scRNA_Labels - Tasic scRNA-seq cell labels
\item seqFISH_Coordinates - Zhu seq-FISH spatial coordinates
\item seqFISH_Counts - Zhu seq-FISH gene counts matrix
\item seqFISH_Labels - Zhu seq-FISH cell labels
}
}
}
\examples{

seqFISH(DataType = "mouse_visual_cortex", modes = "*", version = "2.0.0",
    dry.run = TRUE)

}
\author{
Dario Righelli <dario.righelli \if{html}{\out{<at>}} gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CITEseq.R
\name{CITEseq}
\alias{CITEseq}
\title{CITEseq}
\usage{
CITEseq(
  DataType = c("cord_blood", "peripheral_blood"),
  modes = "*",
  version = "1.0.0",
  dry.run = TRUE,
  filtered = FALSE,
  verbose = TRUE,
  DataClass = c("MultiAssayExperiment", "SingleCellExperiment"),
  ...
)
}
\arguments{
\item{DataType}{\code{character(1)} indicating the identifier of the dataset to
retrieve.  (default "cord_blood")}

\item{modes}{\code{character()} The assay types or modes of data to obtain these
include scADT and scRNA-seq data by default.}

\item{version}{\code{character(1)} Either version '1.0.0' depending on
data version required.}

\item{dry.run}{\code{logical(1)} Whether to return the dataset names before actual
download (default \code{TRUE})}

\item{filtered}{\code{logical(1)} indicating if the returned dataset needs to
have filtered cells.
See Details for additional information about the filtering process.}

\item{verbose}{\code{logical(1)} Whether to show the dataset currently being
(down)loaded (default \code{TRUE})}

\item{DataClass}{either MultiAssayExperiment or SingleCellExperiment
data classes can be returned (default MultiAssayExperiment)}

\item{...}{Additional arguments passed on to the
\link[ExperimentHub]{ExperimentHub-class} constructor}
}
\value{
A single cell multi-modal
\code{\link[MultiAssayExperiment:MultiAssayExperiment-class]{MultiAssayExperiment}}
or informative \code{data.frame} when \code{dry.run} is \code{TRUE}. When \code{DataClass} is
\code{SingleCellExperiment} an object of this class is returned with an RNA
assay as main experiment and other assay(s) as \code{AltExp(s)}.
}
\description{
function assembles data on-the-fly from \code{ExperimentHub} to
provide a
\code{\link[MultiAssayExperiment:MultiAssayExperiment-class]{MultiAssayExperiment}}
container. Actually the \code{dataType} argument provides access to the
available datasets associated to the package.
}
\details{
CITEseq data are a combination of single cell transcriptomics and
about a hundread of cell surface proteins.
Available datasets are:
\itemize{
\item cord_blood: a dataset of single cells of cord blood as
provided in Stoeckius et al. (2017).
\itemize{
\item scRNA_Counts - Stoeckius scRNA-seq gene count matrix
\item scADT - Stoeckius antibody-derived tags (ADT) data
}
\item peripheral_blood: a dataset of single cells of peripheral
blood as provided in Mimitou et al. (2019). We provide two different
conditions controls (CTRL) and Cutaneous T-cell Limphoma (CTCL). Just build
appropriate \code{modes} regex for subselecting the dataset modes.
\itemize{
\item scRNA - Mimitou scRNA-seq gene count matrix
\item scADT - Mimitou antibody-derived tags (ADT) data
\item scHTO - Mimitou Hashtag Oligo (HTO) data
\item TCRab - Mimitou T-cell Receptors (TCR) alpha and beta
available through the object metadata.
\item TCRgd - Mimitou T-cell Receptors (TCR) gamma and delta
available through the object metadata.
}
}

If \code{filtered} parameter is \code{FALSE} (default), the \code{colData} of the returned
object contains multiple columns of \code{logicals} indicating the cells to be
discarded.
In case \code{filtered} is \code{TRUE}, the \code{discard} column is used to filer the
cells.
Column \code{adt.discard} indicates the cells to be discarded computed on the ADT
assay.
Column \code{mito.discard} indicates the cells to be discarded computed on the
RNA assay and mitocondrial genes.
Column \code{discard} combines the previous columns with an \code{OR} operator.
Note that for the \code{peripheral_blood} dataset these three columns are
computed and returned separately for the \code{CTCL} and \code{CTRL} conditions.
In this case the additional \code{discard} column combines the \code{discard.CTCL} and
\code{discard.CTRL} columns with an \code{OR} operator.
Cell filtering has been computed for \code{cord_blood} and \code{peripheral_blood}
datasets following section 12.3 of the Advanced Single-Cell Analysis with
Bioconductor book.
Executed code can be retrieved in the CITEseq_filtering.R script of this
package.
}
\examples{

mae <- CITEseq(DataType="cord_blood", dry.run=FALSE)
experiments(mae)
}
\references{
Stoeckius et al. (2017), Mimitou et al. (2019)
}
\author{
Dario Righelli
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cellGating.R
\name{addCTLabels}
\alias{addCTLabels}
\title{addCTLabels}
\usage{
addCTLabels(
  cd,
  out,
  outname,
  ct,
  mkrcol = "markers",
  ctcol = "celltype",
  overwrite = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{cd}{the \code{colData} \code{DataFrame}}

\item{out}{list data structure returned by \code{getCellGroups}}

\item{outname}{character indicating the name of the out data structure}

\item{ct}{character indicating the celltype to assign in the \code{ctcol}}

\item{mkrcol}{character indicating the cd column to store the markers
indicated by \code{outname} (default is markers)}

\item{ctcol}{character indicating the column in cd to store the cell type
indicated by \code{ct} (default is celltype)}

\item{overwrite}{logical indicating if the cell types have to be overwritten
without checking if detected barcodes were already assigned to other celltypes}

\item{verbose}{logical for having informative messages during the execution}
}
\value{
an updated version of the cd DataFrame
}
\description{
addCTLabels
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SCoPE2.R
\name{SCoPE2}
\alias{SCoPE2}
\title{Single-cell RNA sequencing and proteomics}
\source{
All files are linked from the slavovlab website
\url{https://scope2.slavovlab.net/docs/data}
}
\usage{
SCoPE2(
  DataType = "macrophage_differentiation",
  modes = "*",
  version = "1.0.0",
  dry.run = TRUE,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{DataType}{\code{character(1)} Indicates study that produces this type of
data (default: 'macrophage_differentiation')}

\item{modes}{\code{character()} A wildcard / glob pattern of modes, such as
\code{"rna"}. A wildcard of \code{"*"} will return all modes, that are
transcriptome ("rna") or proteome ("protein") which is the
default.}

\item{version}{\code{character(1)}, currently only version '1.0.0' is
available}

\item{dry.run}{\code{logical(1)} Whether to return the dataset names before actual
download (default \code{TRUE})}

\item{verbose}{\code{logical(1)} Whether to show the dataset currently being
(down)loaded (default \code{TRUE})}

\item{...}{Additional arguments passed on to the
\link[ExperimentHub]{ExperimentHub-class} constructor}
}
\value{
A single cell multi-modal
\code{\link[MultiAssayExperiment:MultiAssayExperiment-class]{MultiAssayExperiment}}
or informative \code{data.frame} when \code{dry.run} is \code{TRUE}
}
\description{
SCoPE2 assembles data on-the-fly from \code{ExperimentHub} to provide
a
\code{\link[MultiAssayExperiment:MultiAssayExperiment-class]{MultiAssayExperiment}}
container. The \code{DataType} argument provides access to the \code{SCoPE2} dataset
as provided by Specht et al. (2020; DOI:
\url{http://dx.doi.org/10.1101/665307}). The article provides more information
about the data acquisition and pre-processing.
}
\details{
The SCoPE2 study combines scRNA-seq (transcriptome) and
single-cell proteomics.
\itemize{
\item macrophage_differentiation: the cells are monocytes that undergo
macrophage differentiation. No annotation is available for the
transcriptome data, but batch and cell type annotations are
available for the proteomics data in the \code{celltype} \code{colData} column.
The transcriptomics and proteomics data were not measured from the same
cells but from a distinct set of cell cultures.
This dataset provides already filtered bad quality cells.
\itemize{
\item scRNAseq1 - single-cell transcriptome (batch 1)
\item scRNAseq2 - single-cell transcriptome (batch 2)
\item scp - single-cell proteomics
}
}
}
\examples{

SCoPE2(DataType = "macrophage_differentiation",
       modes = "*",
       version = "1.0.0",
       dry.run = TRUE)

}
\references{
Specht, Harrison, Edward Emmott, Aleksandra A. Petelski, R.
Gray Huffman, David H. Perlman, Marco Serra, Peter Kharchenko,
Antonius Koller, and Nikolai Slavov. 2020. “Single-Cell
Proteomic and Transcriptomic Analysis of Macrophage
Heterogeneity.” bioRxiv. https://doi.org/10.1101/665307.
}
\seealso{
SingleCellMultiModal-package
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GTseq.R
\name{GTseq}
\alias{GTseq}
\title{Parallel sequencing data of single-cell genomes and transcriptomes}
\source{
\url{https://www.ebi.ac.uk/ena/browser/view/PRJEB9051}
}
\usage{
GTseq(
  DataType = "mouse_embryo_8_cell",
  modes = "*",
  version = "1.0.0",
  dry.run = TRUE,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{DataType}{\code{character(1)} Indicates study that produces this type of
data (default: 'mouse_embryo_8_cell')}

\item{modes}{\code{character()} A wildcard / glob pattern of modes, such as
\code{"*omic"}. A wildcard of \code{"*"} will return all modes including
copy numbers ("genomic") and RNA-seq read counts ("transcriptomic"),
which is the default.}

\item{version}{\code{character(1)} Currently, only version '1.0.0'.}

\item{dry.run}{\code{logical(1)} Whether to return the dataset names before actual
download (default \code{TRUE})}

\item{verbose}{\code{logical(1)} Whether to show the dataset currently being
(down)loaded (default \code{TRUE})}

\item{...}{Additional arguments passed on to the
\link[ExperimentHub:ExperimentHub-class]{ExperimentHub} constructor}
}
\value{
A single cell multi-modal
\link[MultiAssayExperiment:MultiAssayExperiment-class]{MultiAssayExperiment} or
informative \code{data.frame} when \code{dry.run} is \code{TRUE}
}
\description{
GTseq assembles data on-the-fly from \code{ExperimentHub} to provide
a
\code{\link[MultiAssayExperiment:MultiAssayExperiment-class]{MultiAssayExperiment}}
container. The \code{DataType} argument provides access to the
\code{mouse_embryo_8_cell} dataset as obtained from Macaulay et al. (2015).
Protocol information for this dataset is available from Macaulay et al.
(2016). See references.
}
\details{
G&T-seq is a combination of Picoplex amplified gDNA sequencing
(genome) and SMARTSeq2 amplified cDNA sequencing (transcriptome) of the
same cell. For more information, see Macaulay et al. (2015).
* mouse_embryo_8_cell:
this dataset was filtered for bad cells as specified in Macaulay
et al. (2015).
* genomic - integer copy numbers as detected from scDNA-seq
* transcriptomic - raw read counts as quantified from scRNA-seq
}
\section{metadata}{

The \code{MultiAssayExperiment} metadata includes the original function call
that saves the function call and the data version requested.
}

\examples{

GTseq()

}
\references{
Macaulay et al. (2015) G&T-seq: parallel sequencing of single-cell
genomes and transcriptomes. Nat Methods, 12:519–22.

Macaulay et al. (2016) Separation and parallel sequencing of the genomes
and transcriptomes of single cells using G&T-seq. Nat Protoc, 11:2081–103.
}
\seealso{
SingleCellMultiModal-package
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SingleCellMultiModal.R
\name{SingleCellMultiModal}
\alias{SingleCellMultiModal}
\title{Combining Modalities into one MultiAssayExperiment}
\usage{
SingleCellMultiModal(
  DataTypes,
  modes = "*",
  versions = "1.0.0",
  dry.run = TRUE,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{DataTypes}{\code{character()} A vector of data types as indicated in each
individual function by the \code{DataType} parameter. These can be any of
the following: "mouse_gastrulation", "pbmc_10x",
"macrophage_differentiation", "cord_blood", "peripheral_blood",
"mouse_visual_cortex", "mouse_embryo_8_cell"}

\item{modes}{list() A list or CharacterList of modes for each data type
where each element corresponds to one data type.}

\item{versions}{\code{character()} A vector of versions for each DataType. By
default, version \verb{1.0.0} is obtained for all data types.}

\item{dry.run}{\code{logical(1)} Whether to return the dataset names before actual
download (default \code{TRUE})}

\item{verbose}{\code{logical(1)} Whether to show the dataset currently being
(down)loaded (default \code{TRUE})}

\item{...}{Additional arguments passed on to the
\link[ExperimentHub]{ExperimentHub-class} constructor}
}
\value{
A multi-modality \code{MultiAssayExperiment}
}
\description{
Combine multiple single cell modalities into one using the input of the
individual functions.
}
\section{metadata}{

The metadata in the \code{MultiAssayExperiment} contains the original
function call used to generate the object (labeled as \code{call}),
a \code{call_map} which provides traceability of technology functions to
\code{DataType} prefixes, and lastly, R version information as \code{version}.
}

\examples{

SingleCellMultiModal(c("mouse_gastrulation", "pbmc_10x"),
    modes = list(c("acc*", "met*"), "rna"),
    version = c("2.0.0", "1.0.0"), dry.run = TRUE, verbose = TRUE
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scNMT.R
\name{scNMT}
\alias{scNMT}
\title{Single-cell Nucleosome, Methylation and Transcription sequencing}
\source{
\url{http://ftp.ebi.ac.uk/pub/databases/scnmt_gastrulation/}
}
\usage{
scNMT(
  DataType = "mouse_gastrulation",
  modes = "*",
  version = "1.0.0",
  dry.run = TRUE,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{DataType}{\code{character(1)} Indicates study that produces this type of
data (default: 'mouse_gastrulation')}

\item{modes}{\code{character()} A wildcard / glob pattern of modes, such as
\code{"acc*"}. A wildcard of \code{"*"} will return all modes including
Chromatin Accessibilty ("acc"), Methylation ("met"), RNA-seq ("rna")
which is the default.}

\item{version}{\code{character(1)} Either version '1.0.0' or '2.0.0' depending on
data version required (default '1.0.0'). See version section.}

\item{dry.run}{\code{logical(1)} Whether to return the dataset names before actual
download (default \code{TRUE})}

\item{verbose}{\code{logical(1)} Whether to show the dataset currently being
(down)loaded (default \code{TRUE})}

\item{...}{Additional arguments passed on to the
\link[ExperimentHub]{ExperimentHub-class} constructor}
}
\value{
A single cell multi-modal
\code{\link[MultiAssayExperiment:MultiAssayExperiment-class]{MultiAssayExperiment}}
or informative \code{data.frame} when \code{dry.run} is \code{TRUE}
}
\description{
scNMT assembles data on-the-fly from \code{ExperimentHub} to provide
a
\code{\link[MultiAssayExperiment:MultiAssayExperiment-class]{MultiAssayExperiment}}
container. The \code{DataType} argument provides access to the
\code{mouse_gastrulation} dataset as obtained from Argelaguet et al. (2019; DOI:
10.1038/s41586-019-1825-8). Pre-processing code can be seen at
\url{https://github.com/rargelaguet/scnmt_gastrulation}. Protocol
information for this dataset is available at Clark et al. (2018). See the
vignette for the full citation.
}
\details{
scNMT is a combination of RNA-seq (transcriptome) and an adaptation
of Nucleosome Occupancy and Methylation sequencing (NOMe-seq, the
methylome and chromatin accessibility) technologies. For more
information, see Reik et al. (2018) DOI: 10.1038/s41467-018-03149-4
\itemize{
\item mouse_gastrulation - this dataset provides cell quality control filters in
the object \code{colData} starting from version 2.0.0. Additionally, cell types
annotations are provided through the \code{lineage} \code{colData} column.
\itemize{
\item rna - RNA-seq
\item acc_\* - chromatin accessibility
\item met_\* - DNA methylation
\itemize{
\item cgi - CpG islands
\item CTCF - footprints of CTCF binding
\item DHS - DNase Hypersensitive Sites
\item genebody - gene bodies
\item p300 - p300 binding sites
\item promoter - gene promoters
}
}
}

Special thanks to Al J Abadi for preparing the published data in time
for the 2020 BIRS Workshop, see the link here:
\url{https://github.com/BIRSBiointegration/Hackathon/tree/master/scNMT-seq}
}
\section{versions}{

Version '1.0.0' of the scNMT mouse_gastrulation dataset includes all of
the above mentioned assay technologies with filtering of cells based on
quality control metrics. Version '2.0.0' contains all of the cells
without the QC filter and does not contain CTCF binding footprints or
p300 binding sites.
}

\section{metadata}{

The \code{MultiAssayExperiment} metadata includes the original function call
that saves the function call and the data version requested.
}

\examples{

scNMT(DataType = "mouse_gastrulation", modes = "*",
    version = "1.0.0", dry.run = TRUE)

}
\references{
Argelaguet et al. (2019)
}
\seealso{
SingleCellMultiModal-package
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SingleCellMultiModal-package.R
\docType{package}
\name{SingleCellMultiModal-package}
\alias{SingleCellMultiModal-package}
\title{SingleCellMultiModal-package}
\description{
The SingleCellMultiModal package provides a convenient and user-friendly
representation of multi-modal data from project such as \code{scNMT} for mouse
gastrulation.
}
\examples{
help(package = "SingleCellMultiModal")

}
\seealso{
Useful links:
\itemize{
  \item Report bugs at \url{https://github.com/waldronlab/SingleCellMultiModal/issues}
}

}
\author{
\strong{Maintainer}: Marcel Ramos \email{marcel.ramos@sph.cuny.edu} (\href{https://orcid.org/0000-0002-3242-0582}{ORCID})

Authors:
\itemize{
  \item Marcel Ramos \email{marcel.ramos@sph.cuny.edu} (\href{https://orcid.org/0000-0002-3242-0582}{ORCID})
  \item Ricard Argelaguet \email{ricard@ebi.ac.uk}
  \item Dario Righelli \email{dario.righelli@gmail.com}
  \item Kelly Eckenrode \email{kelly.eckenrode@sph.cuny.edu}
  \item Ludwig Geistlinger \email{ludwig_geistlinger@hms.harvard.edu}
  \item Levi Waldron \email{lwaldron.research@gmail.com}
}

Other contributors:
\itemize{
  \item Al Abadi [contributor]
  \item Christophe Vanderaa \email{christophe.vanderaa@uclouvain.be} [contributor]
  \item NCI (GrantNo.: U24CA289073) [funder]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cellGating.R
\name{getCellGroups}
\alias{getCellGroups}
\title{getCellGroups}
\usage{
getCellGroups(mat, adt1 = "CD19", adt2 = "CD3", th1 = 0.2, th2 = 0)
}
\arguments{
\item{mat}{matrix of counts or clr transformed counts for ADT data in CITEseq}

\item{adt1}{character indicating the name of the marker to plot on the x-axis
(default is CD19).}

\item{adt2}{character indicating the name of the marker to plot on the y-axis
(default is CD3).}

\item{th1}{numeric indicating the threshold for the marker on the x-axis
(default is 0.2).}

\item{th2}{numeric indicating the threshold for the marker on the y-axis
(default is 0).}
}
\value{
a list of four different element, each one indicating the quarter
where the thresholds divide the plotting space, in eucledian order I, II,
III, IV quadrant, indicating respectively +/+, +/-, -/+, -/- combinations
for the couples of selected ADTs.
Each element of the list contains two objects, one with the list of detected
barcodes and one indicating the percentage of barcodes falling into that
quadrant.
.
}
\description{
Shows the cells/barcodes in two different plots (scatter and density)
divinding the space in four quadrant indicated by the two thresholds given
as input parameters.
The x/y-axis represent respectively the two ADTs given as input.
It returns a list of one element for each quadrant, each with barcodes and
percentage (see Value section for details).
}
\details{
helps to do manual gating for cell type indentification with CITEseq
or similar data, providing cell markers.
Once identified two interesting markers for a cell type, the user has to
play with the thresholds to identify the cell populations specified by an
uptake (+) o downtake (-) of the couple of markers (ADTs) previously selected.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ontomap.R
\name{ontomap}
\alias{ontomap}
\title{Obtain a map of cell types for each dataset}
\usage{
ontomap(dataset = c("scNMT", "scMultiome", "SCoPE2", "CITEseq", "seqFISH"))
}
\arguments{
\item{dataset}{\code{character()} One of the existing functions within the
package. If missing, a map of all cell types in each function will
be provided.}
}
\value{
A \code{data.frame} of metadata with cell types and ontologies
}
\description{
The \code{ontomap} function provides a mapping of all the cell names across the
all the data sets or for a specified data set.
}
\details{
Note that \code{CITEseq} does not have any cell annotations; therefore, no entries
are present in the \code{ontomap}.
}
\examples{

ontomap(dataset = "scNMT")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CITEseq.R
\name{.CITEseqMaeToSce}
\alias{.CITEseqMaeToSce}
\title{CITEseqMaeToSce}
\usage{
.CITEseqMaeToSce(mae)
}
\arguments{
\item{mae}{a MultiAssayExperiment object with scRNA and/or scADT and/or
scHTO named experiments.}
}
\value{
a SingleCellExperiment object as widely with scRNA data as counts
and scADT, scHTO data as altExps.
If only one modality is present, it has returned as main assay of the SCE.
}
\description{
converts a \code{MultiAssayExperiment} object with CITEseq data into
a \code{SingleCellExperiment} object to be used with already known methods and
packages in literature.

Note that for creating a \code{SingleCellExperiment} object the following function
subsets all the assays present in the \code{MultiAssayExperiment} with only the
common cells across all the modalities.
This could result in a not complete object.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scMultiome.R
\name{scMultiome}
\alias{scMultiome}
\title{Single-cell Multiome ATAC + Gene Expression}
\usage{
scMultiome(
  DataType = "pbmc_10x",
  modes = "*",
  version = "1.0.0",
  format = c("MTX", "HDF5"),
  dry.run = TRUE,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{DataType}{\code{character(1)} Indicates study that produces this type of
data (default: 'mouse_gastrulation')}

\item{modes}{\code{character()} A wildcard / glob pattern of modes, such as
\code{"acc*"}. A wildcard of \code{"*"} will return all modes including
Chromatin Accessibilty ("acc"), Methylation ("met"), RNA-seq ("rna")
which is the default.}

\item{version}{\code{character(1)} Either version '1.0.0' or '2.0.0' depending on
data version required (default '1.0.0'). See version section.}

\item{format}{\code{character(1)} Either MTX or HDF5 data format (default MTX)}

\item{dry.run}{\code{logical(1)} Whether to return the dataset names before actual
download (default \code{TRUE})}

\item{verbose}{\code{logical(1)} Whether to show the dataset currently being
(down)loaded (default \code{TRUE})}

\item{...}{Additional arguments passed on to the
\link[ExperimentHub]{ExperimentHub-class} constructor}
}
\value{
A 10X PBMC \code{MultiAssayExperiment} object
}
\description{
10x Genomics Multiome technology enables simultaneous profiling
of the transcriptome (using 3’ gene expression) and epigenome
(using ATAC-seq) from single cells to
deepen our understanding of how genes are expressed and regulated across
different cell types. Data prepared by Ricard Argelaguet.
}
\details{
Users are able to choose from either an \code{MTX} or \code{HDF5} file format
as the internal data representation. The \code{MTX} (Matrix Market) format
allows users to load a sparse \code{dgCMatrix} representation. Choosing \code{HDF5}
gives users a sparse \code{HDF5Array} class object.
* pbmc_10x: 10K Peripheral Blood Mononuclear Cells provided by
\href{https://support.10xgenomics.com/single-cell-multiome-atac-gex/datasets}{10x Genomics website}
Cell quality control filters are available in the object \code{colData}
together with the \code{celltype} annotation labels.
}
\examples{

scMultiome(DataType = "pbmc_10x", modes = "*", dry.run = TRUE)

}
